@@ -1,37 +1,81 @@
 package Mail::Milter::Authentication;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: A Perl Mail Authentication Milter
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Net::Server::PreFork';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use English qw{ -no_match_vars };
-use ExtUtils::Installed;
-use JSON;
-use Mail::Milter::Authentication::Config qw{ get_config };
-use Mail::Milter::Authentication::Constants qw{ :all };
 use Mail::Milter::Authentication::Handler;
 use Mail::Milter::Authentication::Metric;
 use Mail::Milter::Authentication::Protocol::Milter;
 use Mail::Milter::Authentication::Protocol::SMTP;
-use Module::Load;
-use Module::Loaded;
+use Email::Sender::Simple qw(try_to_sendmail);
+use Email::Simple::Creator;
+use Email::Simple;
+use ExtUtils::Installed;
+use Log::Dispatchouli;
 use Net::DNS::Resolver;
 use Net::IP;
 use Proc::ProcessTable;
-use Sys::Syslog qw{:standard :macros};
-
 use vars qw(@ISA);
 
-sub _warn {
-    my ( $msg ) = @_;
+
+{
+    my $LOGGER;
+    sub logger {
+
+        return $LOGGER if $LOGGER;
+        my $config = get_config();
+
+        my $MYARGS = {
+            'ident' => $Mail::Milter::Authentication::Config::IDENT,
+            'to_stderr' => 0, # handled elsewhere
+            'log_pid' => 1,
+            'facility' => LOG_MAIL,
+        };
+        if ( exists $config->{ 'log_dispatchouli' } ) {
+            $MYARGS = $config->{ 'log_dispatchouli' };
+        }
+
+        $LOGGER = Log::Dispatchouli->new( $MYARGS );
+        $LOGGER->log('Logging instantiated');
+        return $LOGGER;
+    }
+}
+
+sub _warn($msg) {
     my @parts = split "\n", $msg;
     foreach my $part ( @parts ) {
         next if $part eq q{};
         print STDERR scalar(localtime) . ' ' . $Mail::Milter::Authentication::Config::IDENT . "[$PID] $part\n";
     }
-    return;
 }
 
+
+sub preload_modules($self,$from,$matching) {
+    my $installed = ExtUtils::Installed->new( 'skip_cwd' => 1 );
+    my $path_matching = $matching;
+    $path_matching =~ s/::/\//g;
+    foreach my $module ( grep { /$from/ } $installed->modules() ) {
+        FILE:
+        foreach my $file ( grep { /$path_matching\/\w+\.pm$/ } $installed->files( $module ) ) {
+            next FILE if ! -e $file;
+            my ( $module ) = reverse split '/', $file;
+            $module =~ s/\.pm$//;
+            $module = join( '::', $matching, $module );
+            if ( ! is_loaded( $module ) ) {
+                $self->logdebug( "Preloading Module $module" );
+                load $module;
+            }
+            else {
+                $self->logdebug( "Preloading Module $module already loaded" );
+            }
+        }
+    }
+}
+
+
 sub get_installed_handlers {
     my @installed_handlers;
     my $installed = ExtUtils::Installed->new( 'skip_cwd' => 1 );
@@ -47,12 +91,28 @@ sub get_installed_handlers {
     return \@installed_handlers;
 }
 
-sub pre_loop_hook {
-    my ( $self ) = @_;
 
+sub write_to_log_hook($self,$priority,$line,@) {
+    my $log_priority = $priority == 0 ? 'debug'
+                     : $priority == 1 ? 'info'
+                     : $priority == 2 ? 'notice'
+                     : $priority == 3 ? 'warning'
+                     : $priority == 4 ? 'error'
+                     : 'debug';
+    logger()->log( { 'level' => $log_priority }, $line );
+}
+
+
+sub idle_loop_hook($self,@) {
+    $self->{'metric'}->master_metric_update( $self );
+}
+
+
+sub pre_loop_hook($self,@) {
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':master';
 
-    $self->{'metric'} = Mail::Milter::Authentication::Metric->new();
+    $self->preload_modules( 'Net::DNS', 'Net::DNS::RR' );
+    $self->{'metric'} = Mail::Milter::Authentication::Metric->new($self);
 
     # Load handlers
     my $config = get_config();
@@ -91,16 +151,13 @@ sub pre_loop_hook {
         open( STDERR, '>>', $config->{'error_log'} ) || die "Cannot open errlog [$!]";
         open( STDOUT, '>>', $config->{'error_log'} ) || die "Cannot open errlog [$!]";
     }
-
-    return;
-
 }
 
-sub run_n_children_hook {
-    my ( $self ) = @_;
 
+sub run_n_children_hook($self,@) {
     # Load handlers
     my $config = get_config();
+    $self->{metric}->re_register_metrics;
     foreach my $name ( @{$config->{'load_handlers'}} ) {
 
         my $package = "Mail::Milter::Authentication::Handler::$name";
@@ -110,12 +167,11 @@ sub run_n_children_hook {
         }
 
     }
-
-    return;
 }
 
-sub child_init_hook {
-    my ( $self ) = @_;
+
+sub child_init_hook($self,@) {
+    srand();
 
     my $config = get_config();
     $self->{'config'} = $config;
@@ -136,7 +192,6 @@ sub child_init_hook {
     my $base;
     if ( $config->{'protocol'} eq 'milter' ) {
         $base = 'Mail::Milter::Authentication::Protocol::Milter';
-
     }
     elsif ( $config->{'protocol'} eq 'smtp' ) {
         $base = 'Mail::Milter::Authentication::Protocol::SMTP';
@@ -176,29 +231,28 @@ sub child_init_hook {
     $self->{'object_maker'}   = $object_maker;
 
     $self->setup_handlers();
+    $self->{'metric'}->set_versions( $self );
 
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':waiting(0)';
-    return;
 }
 
-sub child_finish_hook {
-    my ($self) = @_;
+
+sub child_finish_hook($self,@) {
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':exiting';
     $self->loginfo( "Child process $PID shutting down" );
-    $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
-    $self->{'handler'}->{'_Handler'}->metric_send();
+    eval {
+        $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
+    };
     $self->destroy_objects();
-    return;
 }
 
-sub pre_server_close_hook {
-    my ($self) = @_;
+
+sub pre_server_close_hook($self,@) {
     $self->loginfo( 'Server closing down' );
-    return;
 }
 
-sub get_client_proto {
-    my ( $self ) = @_;
+
+sub get_client_proto($self,@) {
     my $socket = $self->{server}{client};
     if ($socket->isa("Net::Server::Proto")) {
         my $proto = $socket->NS_proto;
@@ -219,30 +273,28 @@ sub get_client_proto {
     }
 
     $self->logerror( "Could not determine connection protocol: " . ref($socket) );
-
-    return;
 }
 
-sub get_client_port {
-    my ( $self ) = @_;
+
+sub get_client_port($self,@) {
     my $socket = $self->{server}{client};
     return $socket->sockport();
 }
 
-sub get_client_host {
-    my ( $self ) = @_;
+
+sub get_client_host($self,@) {
     my $socket = $self->{server}{client};
     return $socket->sockhost();
 }
 
-sub get_client_path {
-    my ( $self ) = @_;
+
+sub get_client_path($self,@) {
     my $socket = $self->{server}{client};
     return $socket->hostpath();
 }
 
-sub get_client_details {
-    my ( $self ) = @_;
+
+sub get_client_details($self,@) {
     my $proto = lc $self->get_client_proto();
     if ( $proto eq 'tcp' ) {
         return 'inet:' . $self->get_client_port();
@@ -250,44 +302,10 @@ sub get_client_details {
     elsif ( $proto eq 'unix' ) {
         return 'unix:' . $self->get_client_path();
     }
-    return;
 }
 
-sub child_is_talking_hook {
-    my ( $self, $socket ) = @_;
 
-    my $request;
-    my $raw_request;
-
-    eval {
-        $raw_request = <$socket>;
-        return if ! $raw_request;
-        $request = decode_json( $raw_request );
-    };
-    if ( my $error = $@ ) {
-        warn "Error $error reading from child";
-        return;
-    }
-    else {
-
-        if ( ! $request ) {
-            warn "Ignoring Invalid child request: $raw_request;";
-            return;
-        }
-        $request =~ s/[\n\r]+$//;
-        if ( $request->{ 'method' } eq 'METRIC.GET' ) {
-            $self->{'metric'}->master_handler( $request, $socket, $self );
-        }
-        if ( $request->{ 'method' } eq 'METRIC.COUNT' ) {
-            $self->{'metric'}->master_handler( $request, $socket, $self );
-        }
-    }
-
-    return;
-}
-
-sub process_request {
-    my ( $self ) = @_;
+sub process_request($self,@) {
     my $config = $self->{'config'};
 
     my $metric_type;
@@ -325,13 +343,10 @@ sub process_request {
 
     my $count = $self->{'count'};
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':waiting(' . $count . ')';
-    return;
 }
 
 
-sub process_main {
-    my ( $self ) = @_;
-
+sub process_main($self,@) {
     $self->{'count'}++;
     my $count = $self->{'count'};
     my $config = $self->{'config'};
@@ -340,16 +355,19 @@ sub process_main {
     $self->logdebug( 'Processing request ' . $self->{'count'} );
     $self->{'socket'} = $self->{'server'}->{'client'};
 
+    $self->{'tracelog'} = [];
+
     $self->protocol_process_request();
 
     # Call close callback
     $self->{'handler'}->{'_Handler'}->top_close_callback();
     if ( $self->{'handler'}->{'_Handler'}->{'exit_on_close'} ) {
-        $self->{'metric'}->send( $self );
-        $self->fatal('exit_on_close requested');
+        my $error = $self->{'handler'}->{'_Handler'}->{'exit_on_close_error'} // 'no reason given';
+        $self->send_exception_email($error);
+        $self->fatal('exit_on_close requested - '.$error);
     }
 
-    $self->{'metric'}->send( $self );
+    $self->{'tracelog'} = [];
 
     if ( $config->{'debug'} ) {
         my $process_table = Proc::ProcessTable->new();
@@ -368,15 +386,107 @@ sub process_main {
     delete $self->{'handler'}->{'_Handler'}->{'return_code'};
     delete $self->{'socket'};
     $self->logdebug( 'Request processing completed' );
-    return;
 }
 
 
+sub send_exception_email {
+    my ( $self, $error ) = @_;
+
+    if ( $self->{'handler'}->{'_Handler'}->{'suppress_error_emails'} ) {
+        return;
+    }
+
+    my $config         = get_config();
+    my $errors_to      = $config->{'errors_to'} || return;
+    my $errors_from    = $config->{'errors_from'} || return;
+    my $errors_headers = $config->{'errors_headers'} || {};
+
+    my $email = "Authentication Milter " . $Mail::Milter::Authentication::Config::IDENT . " Error\n\n";
+    $email .= "$error\n\n";
+
+    $email .= "Child PID: $PID\n\n";
+
+    $email .= "Log:\n";
+    $email .= join( "\n", $self->{'tracelog'}->@* );
+    $email .= "\n\n";
+
+    $email .= "Processes Running\n";
+    my $ppid = $self->{'server'}->{'ppid'};
+    my $process_table = Proc::ProcessTable->new;
+    foreach my $process ( $process_table->table->@* ) {
+        next if ! ( $process->pid == $ppid || $process->ppid == $ppid );
+        my $pid = eval{ $process->pid } // '';
+        my $cmndline = eval{ $process->cmndline } // '';
+        my $size   = eval{ $process->size } // '';
+        my $rss    = eval{ $process->rss } // '';
+        my $pctmem = eval{ $process->pctmem } // '';
+        my $pctcpu = eval{ $process->pctcpu } // '';
+        $email .= "pid:$pid, $cmndline, size:$size, mem:$rss ($pctmem%), cpu: $pctcpu%\n";
+    }
+
+    my @headers;
+    foreach my $key ( sort keys $errors_headers->%* ) {
+        push @headers, $key => $errors_headers->{$key};
+    }
+    push @headers, to => $errors_to;
+    push @headers, From => $errors_from;
+    push @headers, Subject => 'Authentication Milter Error';
+    push @headers, 'X-Authentication-Milter-Error' => 'Generated Error Report';
+
+    my $emailer = Email::Simple->create(
+        header => \@headers,
+        body => $email,
+    );
+    try_to_sendmail($emailer);
+
+}
 
 
+sub send_panic_email {
+    my ( $error ) = @_;
 
-sub get_valid_pid {
-    my ( $pid_file ) = @_;
+    my $config         = get_config();
+    my $errors_to      = $config->{'errors_to'} || return;
+    my $errors_from    = $config->{'errors_from'} || return;
+    my $errors_headers = $config->{'errors_headers'} || {};
+
+    my $email = "Authentication Milter " . $Mail::Milter::Authentication::Config::IDENT . " Error\n\n";
+    $email .= "$error\n\n";
+
+    $email .= "PID: $PID\n\n";
+
+    $email .= "Processes Running\n";
+    my $process_table = Proc::ProcessTable->new;
+    foreach my $process ( $process_table->table->@* ) {
+        next if ! ( $process->pid == $PID || $process->ppid == $PID );
+        my $pid = eval{ $process->pid } // '';
+        my $cmndline = eval{ $process->cmndline } // '';
+        my $size   = eval{ $process->size } // '';
+        my $rss    = eval{ $process->rss } // '';
+        my $pctmem = eval{ $process->pctmem } // '';
+        my $pctcpu = eval{ $process->pctcpu } // '';
+        $email .= "pid:$pid, $cmndline, size:$size, mem:$rss ($pctmem%), cpu: $pctcpu%\n";
+    }
+
+    my @headers;
+    foreach my $key ( sort keys $errors_headers->%* ) {
+        push @headers, $key => $errors_headers->{$key};
+    }
+    push @headers, to => $errors_to;
+    push @headers, From => $errors_from;
+    push @headers, Subject => 'Authentication Milter Error';
+    push @headers, 'X-Authentication-Milter-Error' => 'Generated Error Report';
+
+    my $emailer = Email::Simple->create(
+        header => \@headers,
+        body => $email,
+    );
+    try_to_sendmail($emailer);
+
+}
+
+
+sub get_valid_pid($pid_file) {
     if ( ! $pid_file ) {
         return undef; ## no critic
     }
@@ -393,7 +503,7 @@ sub get_valid_pid {
     my $found_pid  = 0;
 
     my $process_table = Proc::ProcessTable->new();
-    foreach my $process ( @{$process_table->table} ) {
+    foreach my $process ( $process_table->table->@* ) {
         if ( $process->pid == $self_pid ) {
             if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':control' ) {
                 $found_self = 1;
@@ -419,9 +529,10 @@ sub get_valid_pid {
     return undef; ## no critic
 }
 
+
 sub find_process {
     my $process_table = Proc::ProcessTable->new();
-    foreach my $process ( @{$process_table->table} ) {
+    foreach my $process ( $process_table->table->@* ) {
         if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':master' ) {
             return $process->pid;
         }
@@ -429,8 +540,8 @@ sub find_process {
     return undef; ## no critic
 }
 
-sub control {
-    my ( $args ) = @_;
+
+sub control($args) {
     my $pid_file = $args->{'pid_file'};
     my $command  = $args->{'command'};
 
@@ -477,19 +588,15 @@ sub control {
     else {
         die 'unknown command';
     }
-
-    return;
 }
 
-sub start {
-    my ($args)     = @_;
 
+sub start($args) {
     local $SIG{__WARN__} = sub {
         foreach my $msg ( @_ ) {
-            syslog( LOG_ERR, "Warning: $msg" );
+            logger()->log( { level => 'warning' }, "Warning: $msg" );
             _warn( "Warning: $msg" );
         }
-        return;
     };
 
     my $config                 = get_config();
@@ -509,6 +616,12 @@ sub start {
 
     $srvargs{'no_client_stdout'} = 1;
 
+    # Early redirection to log file if possible
+    if ( $config->{'error_log'} ) {
+        open( STDERR, '>>', $config->{'error_log'} ) || die "Cannot open errlog [$!]";
+        open( STDOUT, '>>', $config->{'error_log'} ) || die "Cannot open errlog [$!]";
+    }
+
     if ( $args->{'daemon'} ) {
         if ( $EUID == 0 ) {
             _warn(
@@ -533,12 +646,6 @@ sub start {
     $srvargs{'min_spare_servers'} = $min_spare_children;
     $srvargs{'max_spare_servers'} = $max_spare_children;
 
-    $srvargs{'log_file'}          = 'Sys::Syslog';
-    $srvargs{'syslog_facility'}   = LOG_MAIL;
-    $srvargs{'syslog_ident'}      = $Mail::Milter::Authentication::Config::IDENT;
-    $srvargs{'syslog_logopt'}     = 'pid';
-    $srvargs{'syslog_logsock'}    = 'native';
-
     if ( $EUID == 0 ) {
         my $user  = $config->{'runas'};
         my $group = $config->{'rungroup'};
@@ -667,7 +774,7 @@ sub start {
                 'port'  => $path,
                 'proto' => 'unix',
             };
-            $srvargs{'child_communication'} = 1;
+            $srvargs{'child_communication'} = 0;
 
             if ($umask) {
                 umask ( oct( $umask ) );
@@ -722,32 +829,38 @@ sub start {
         $error = 'unknown error' if ! $error;
         _warn "Server failed: $error";
 
+        # We exited abnormally, try and clean up
+        eval{ __PACKAGE__->close_children };
+        eval{ __PACKAGE__->post_child_cleanup_hook };
+        eval{ __PACKAGE__->shutdown_sockets };
+        sleep 10;
+
         if ( scalar @start_times >= 4 ) {
             if ( $start_times[3] > ( time() - 120 ) ) {
+                eval{ send_panic_email("Error: $error - Abandoning") };
                 _warn "Abandoning automatic restart: too many restarts in a short time";
                 last;
             }
         }
 
+        eval{ send_panic_email("Error: $error - Attempting automatic restart") };
         _warn "Attempting automatic restart";
         sleep 10;
     }
     _warn "Server exiting abnormally";
     die;
-
-    return; ## no critic
 }
 
 ##### Protocol methods
 
-sub fatal {
-    my ( $self, $error ) = @_;
+
+sub fatal($self,$error) {
     $self->logerror( "Child process $PID shutting down due to fatal error: $error" );
     die "$error\n";
 }
 
-sub fatal_global {
-    my ( $self, $error ) = @_;
+
+sub fatal_global($self,$error) {
     my $ppid = $self->{'server'}->{'ppid'};
     if ( $ppid == $PID ) {
         $self->logerror( "Global shut down due to fatal error: $error" );
@@ -759,27 +872,23 @@ sub fatal_global {
     die "$error\n";
 }
 
-sub setup_handlers {
-    my ( $self ) = @_;
 
+sub setup_handlers($self) {
     $self->logdebug( 'setup objects' );
     my $handler = Mail::Milter::Authentication::Handler->new( $self );
     $self->{'handler'}->{'_Handler'} = $handler;
 
     $handler->metric_count( 'forked_children_total', {}, 1 );
-    $handler->metric_send();
 
     my $config = $self->{'config'};
-    foreach my $name ( @{$config->{'load_handlers'}} ) {
+    foreach my $name ( $config->{'load_handlers'}->@* ) {
         $self->setup_handler( $name );
     }
     $self->sort_all_callbacks();
-    return;
 }
 
-sub load_handler {
-    my ( $self, $name ) = @_;
 
+sub load_handler($self,$name) {
     ## TODO error handling here
     $self->logdebug( "Load Handler $name" );
 
@@ -791,12 +900,10 @@ sub load_handler {
             $self->fatal_global('Could not load handler ' . $name . ' : ' . $error);
         }
     }
-    return;
 }
 
-sub setup_handler {
-    my ( $self, $name ) = @_;
 
+sub setup_handler($self,$name) {
     ## TODO error handling here
     $self->logdebug( "Instantiate Handler $name" );
 
@@ -804,15 +911,14 @@ sub setup_handler {
     my $object = $package->new( $self );
     $self->{'handler'}->{$name} = $object;
 
-    foreach my $callback ( qw { setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
         if ( $object->can( $callback . '_callback' ) ) {
             $self->register_callback( $name, $callback );
         }
     }
-
-    return;
 }
 
+
 sub destroy_handler {
     # Unused!
     my ( $self, $name ) = @_;
@@ -820,30 +926,26 @@ sub destroy_handler {
     delete $self->{'handler'}->{$name}->{'thischild'};
     # Remove reference to handler
     delete $self->{'handler'}->{$name};
-    return;
 }
 
-sub register_callback {
-    my ( $self, $name, $callback ) = @_;
+
+sub register_callback($self,$name,$callback) {
     $self->logdebug( "Register Callback $name:$callback" );
     if ( ! exists $self->{'callbacks'}->{$callback} ) {
         $self->{'callbacks'}->{$callback} = [];
     }
     push @{ $self->{'callbacks'}->{$callback} }, $name;
-    return;
 }
 
-sub sort_all_callbacks {
-    my ($self) = @_;
-    foreach my $callback ( qw { setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+
+sub sort_all_callbacks($self) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
         $self->sort_callbacks( $callback );
     }
-    return;
 }
 
-sub sort_callbacks {
-    my ( $self, $callback ) = @_;
 
+sub sort_callbacks($self,$callback) {
     if ( ! exists $self->{'callbacks'}->{$callback} ) {
         $self->{'callbacks'}->{$callback} = [];
     }
@@ -862,31 +964,48 @@ sub sort_callbacks {
 
     my @todo = sort @{$callbacks_ref};
     my $todo_count = scalar @todo;
+
+    # Process requirements
+    my $requirements = {};
+    foreach my $item ( @todo ) {
+        $requirements->{ $item } = [];
+        my $handler = $self->{'handler'}->{ $item };
+        my $requires_method = $callback . '_requires';
+        if ( $handler->can( $requires_method ) ) {
+            my $requires = $handler->$requires_method;
+            foreach my $require ( $requires->@* ) {
+                push @{ $requirements->{ $item } }, $require;
+            }
+        }
+    }
+    foreach my $item ( @todo ) {
+        my $handler = $self->{'handler'}->{ $item };
+        my $requires_method = $callback . '_required_before';
+        if ( $handler->can( $requires_method ) ) {
+            my $requires = $handler->$requires_method;
+            foreach my $require ( $requires->@* ) {
+                push @{ $requirements->{ $require } }, $item;
+            }
+        }
+    }
+
     while ( $todo_count ) {
         my @defer;
         foreach my $item ( @todo ) {
-            my $handler = $self->{'handler'}->{ $item };
-            my $requires_method = $callback . '_requires';
-            if ( $handler->can( $requires_method ) ) {
-                my $requires_met = 1;
-                my $requires = $handler->$requires_method;
-                foreach my $require ( @{ $requires } ) {
-                    if ( ! exists $added->{$require} ) {
-                        $requires_met = 0;
-                    }
-                }
-                if ( $requires_met == 1 ) {
-                    push @order, $item;
-                    $added->{$item} = 1;
-                }
-                else {
-                    push @defer, $item;
+            my $requires_met = 1;
+            my $requires = $requirements->{ $item };
+            foreach my $require ( $requires->@* ) {
+                if ( ! exists $added->{$require} ) {
+                    $requires_met = 0;
                 }
             }
-            else {
+            if ( $requires_met == 1 ) {
                 push @order, $item;
                 $added->{$item} = 1;
             }
+            else {
+                push @defer, $item;
+            }
         }
 
         my $defer_count = scalar @defer;
@@ -898,24 +1017,22 @@ sub sort_callbacks {
     }
 
     $self->{'callbacks_list'}->{$callback} = \@order;
-    return;
 }
 
-sub destroy_objects {
-    my ( $self ) = @_;
+
+sub destroy_objects($self) {
     $self->logdebug ( 'destroy objects' );
     my $handler = $self->{'handler'}->{'_Handler'};
     if ( $handler ) {
         $handler->destroy_all_objects();
         my $config = $self->{'config'};
-        foreach my $name ( @{$config->{'load_handlers'}} ) {
+        foreach my $name ( $config->{'load_handlers'}->@* )  {
             $self->destroy_handler( $name );
         }
         delete $self->{'handler'}->{'_Handler'}->{'config'};
         delete $self->{'handler'}->{'_Handler'}->{'thischild'};
         delete $self->{'handler'}->{'_Handler'};
     }
-    return;
 }
 
 
@@ -923,8 +1040,8 @@ sub destroy_objects {
 
 ## Logging
 
-sub get_queue_id {
-    my ( $self ) = @_;
+
+sub get_queue_id($self) {
     my $queue_id;
 
     if ( exists ( $self->{'smtp'} ) ) {
@@ -939,8 +1056,8 @@ sub get_queue_id {
     return $queue_id;
 }
 
-sub enable_extra_debugging {
-    my ($self) = @_;
+
+sub enable_extra_debugging($self) {
     my $config = $self->{'config'} || get_config();
     $config->{'logtoerr'} = 1;
     $config->{'debug'}    = 1;
@@ -948,119 +1065,238 @@ sub enable_extra_debugging {
     $self->logerror( 'Extra debugging enabled. Child will exit on close.' );
     # We don't want to persist this, so force an exit on close state.
     $self->{'handler'}->{'_Handler'}->{'exit_on_close'} = 1;
-    return;
 }
 
-sub extra_debugging {
-    my ($self,$line) = @_;
+
+sub extra_debugging($self,$line) {
     if ( $self->{'extra_debugging'} ) {
         $self->logerror( $line );
     }
-    return;
 }
 
-sub logerror {
-    my ($self,$line) = @_;
+
+sub logerror($self,$line) {
     my $config = $self->{'config'} || get_config();
     if ( my $queue_id = $self->get_queue_id() ) {
         $line = $queue_id . ': ' . $line;
     }
     _warn( $line ) if $config->{'logtoerr'};
-    syslog( LOG_ERR, $line );
-    return;
+    logger()->log( { 'level' => 'error' }, $line );
 }
 
-sub loginfo {
-    my ($self,$line) = @_;
+
+sub loginfo($self,$line) {
     my $config = $self->{'config'} || get_config();
     if ( my $queue_id = $self->get_queue_id() ) {
         $line = $queue_id . ': ' . $line;
     }
     _warn( $line ) if $config->{'logtoerr'};
-    syslog( LOG_INFO, $line );
-    return;
+    logger()->log( { 'level' => 'info' }, $line );
 }
 
-sub logdebug {
-    my ($self,$line) = @_;
+
+sub logdebug($self,$line) {
     my $config = $self->{'config'} || get_config();
     if ( my $queue_id = $self->get_queue_id() ) {
         $line = $queue_id . ': ' . $line;
     }
     if ( $config->{'debug'} ) {
         _warn( $line ) if $config->{'logtoerr'};
-        syslog( LOG_DEBUG, $line );
+        logger()->log( { 'level' => 'debug' }, $line );
     }
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-A Perl Implementation of email authentication standards rolled up into a single easy to use milter.
+version 2.20200329.1
 
 =head1 SYNOPSIS
 
 Subclass of Net::Server::PreFork for bringing up the main server process for authentication_milter.
 
+This class handles the server aspects of Authentication Milter.
+
+For individual Protocol handling please see the Mail::Milter::Authentication::Protocol::* classes
+
+For request handling please see Mail::Milter::Authentication::Handler
+
 Please see Net::Server docs for more detail of the server code.
 
 Please see the output of 'authentication_milter --help' for usage help.
 
-=head1 FUNCTIONS
+=head1 DESCRIPTION
 
-=over
-
-=item I<get_installed_handlers()>
-
-Return an array ref of installed handler modules.
-
-=back
+A Perl Implementation of email authentication standards rolled up into a single easy to use milter.
 
 =head1 METHODS
 
-=over
+=head2 preload_modules( $from, $matching )
 
-=item I<child_init_hook()>
+Preload (pre-fork) lazy loading modules.
 
-Hook which runs after forking, sets up per process items.
+Takes a Package Name and a Base module, and loads all modules which match.
 
-=item I<run_n_children_hook()>
+=head2 I<write_to_log_hook()>
+
+Hook which runs to write logs
+
+=head2 I<idle_loop_hook()>
+
+Hook which runs in the master periodically.
+
+=head2 I<pre_loop_hook()>
+
+Hook which runs in the master before looping.
+
+=head2 I<run_n_children_hook()>
 
 Hook which runs in parent before it forks children.
 
-=item I<child_is_talking_hook( $sock )>
+=head2 I<child_init_hook()>
 
-Hook which runs when a child wishes to communicate with the parent.
+Hook which runs after forking, sets up per process items.
 
-=item I<process_request()>
+=head2 I<child_finish_hook()>
+
+Hook which runs when the child is about to finish.
+
+=head2 I<pre_server_close_hook()>
+
+Hook which runs before the server closes.
+
+=head2 I<get_client_proto()>
+
+Get the protocol of the connecting client.
+
+=head2 I<get_client_port()>
+
+Get the port of the connecting client.
+
+=head2 I<get_client_host()>
+
+Get the host of the connecting client.
+
+=head2 I<get_client_path()>
+
+Get the path of the connecting client.
+
+=head2 I<get_client_details()>
+
+Get the details of the connecting client.
+
+=head2 I<process_request()>
 
 Hook which runs for each request, passes control to metrics handler or process_main as appropriate.
 
-=item I<process_main()>
+=head2 I<process_main()>
 
 Method which runs for each request, sets up per request items and processes the request.
 
-=item I<control($command)>
+=head2 I<send_exception_email()>
 
-Run a daemon command.  Command can be one of start/restart/stop/status.
+Send an email to the administrator with details of a problem.
 
-=item I<find_process()>
+=head2 I<fatal($error)>
 
-Search the process table for an authentication_milter master process
+Log a fatal error and die in child
 
-=item I<get_valid_pid($pid_file)>
+=head2 I<fatal_global($error)>
+
+Log a fatal error and die in child and parent
+
+=head2 I<setup_handlers()>
+
+Setup the Handler objects.
+
+=head2 I<load_handler( $name )>
+
+Load the $name Handler module
+
+=head2 I<setup_handler( $name )>
+
+Setup the $name Handler object
+
+=head2 I<destroy_handler( $name )>
+
+Remove the $name Handler
+
+=head2 I<register_callback( $name, $callback )>
+
+Register the specified callback
+
+=head2 I<sort_all_callbacks()>
+
+Sort the callbacks into the order in which they must be called
+
+=head2 I<sort_callbacks( $callback )>
+
+Sort the callbacks for the $callback callback into the right order
+
+=head2 I<destroy_objects()>
+
+Remove references to all objects
+
+=head2 I<get_queue_id()>
+
+Return the queue ID (for logging) if possible.
+
+=head2 I<enable_extra_debugging()>
+
+Turn on extra debugging mode, will cause child to exit on close.
+
+=head2 I<extra_debugging( $line )>
+
+Cause $line to be written to log if extra debugging mode is enabled.
+
+=head2 I<logerror( $line )>
+
+Log to the error log.
+
+=head2 I<loginfo( $line )>
+
+Log to the info log.
+
+=head2 I<logdebug( $line )>
+
+Log to the debug log.
+
+=head1 FUNCTIONS
+
+=head2 I<get_installed_handlers()>
+
+Return an array ref of installed handler modules.
+
+=head2 I<send_panic_email()>
+
+Send an email to the administrator with details of a problem.
+
+Called from the parent process if the server exits.
+
+=head2 I<get_valid_pid($pid_file)>
 
 Given a pid file, check for a valid process ID and return if valid.
 
-=item I<start($hashref)>
+=head2 I<find_process()>
+
+Search the process table for an authentication_milter master process
+
+=head2 I<control($command)>
+
+Run a daemon command.  Command can be one of start/restart/stop/status.
+
+=head2 I<start($hashref)>
 
 Start the server. This method does not return.
 
@@ -1069,129 +1305,15 @@ Start the server. This method does not return.
         'daemon'     => 1/0,                   # Daemonize process?
     }
 
-=item I<fatal($error)>
+=head1 AUTHOR
 
-Log a fatal error and die in child
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=item I<fatal_global($error)>
+=head1 COPYRIGHT AND LICENSE
 
-Log a fatal error and die in child and parent
+This software is copyright (c) 2020 by Marc Bradshaw.
 
-=item I<setup_handlers()>
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-Setup the Handler objects.
-
-=item I<load_handler( $name )>
-
-Load the $name Handler module
-
-=item I<setup_handler( $name )>
-
-Setup the $name Handler object
-
-=item I<destroy_handler( $name )>
-
-Remove the $name Handler
-
-=item I<register_callback( $name, $callback )>
-
-Register the specified callback
-
-=item I<sort_all_callbacks()>
-
-Sort the callbacks into the order in which they must be called
-
-=item I<sort_callbacks( $callback )>
-
-Sort the callbacks for the $callback callback into the right order
-
-=item I<destroy_objects()>
-
-Remove references to all objects
-
-=item I<get_queue_id()>
-
-Return the queue ID (for logging) if possible.
-
-=item I<logerror( $line )>
-
-Log to the error log.
-
-=item I<loginfo( $line )>
-
-Log to the info log.
-
-=item I<logdebug( $line )>
-
-Log to the debug log.
-
-=item I<enable_extra_debugging()>
-
-Turn on extra debugging mode, will cause child to exit on close.
-
-=item I<extra_debugging( $line )>
-
-Cause $line to be written to log if extra debugging mode is enabled.
-
-=item I<child_finish_hook()>
-
-Hook which runs when the child is about to finish.
-
-=item I<get_client_details()>
-
-Get the details of the connecting client.
-
-=item I<get_client_path()>
-
-Get the path of the connecting client.
-
-=item I<get_client_port()>
-
-Get the port of the connecting client.
-
-=item I<get_client_host()>
-
-Get the host of the connecting client.
-
-=item I<get_client_proto()>
-
-Get the protocol of the connecting client.
-
-=item I<pre_loop_hook()>
-
-Hook which runs in the master before looping.
-
-=item I<pre_server_close_hook()>
-
-Hook which runs before the server closes.
-
-=back
-
-=head1 DEPENDENCIES
-
-  English
-  JSON
-  Mail::DKIM
-  Mail::DMARC
-  Mail::SPF
-  MIME::Base64
-  Module::Load
-  Module::Loaded
-  Net::DNS
-  Net::IP
-  Net::Server
-  Pod::Usage
-  Proc::ProcessTable
-  Sys::Hostname
-  Sys::Syslog
-
-=head1 AUTHORS
-
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+=cut
