@@ -1,23 +1,30 @@
 package Mail::Milter::Authentication::Config;
+# ABSTRACT: Load config files for Authentication Milter
+use 5.20.0;
 use strict;
 use warnings;
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Mail::Milter::Authentication;
+##use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Common configuration handling
+our $VERSION = '2.20200329.1'; # VERSION
+use English;
+use JSON;
+use TOML;
 use Module::Load;
+use Module::Loaded;
 
 use Exporter qw{ import };
 our @EXPORT_OK = qw{
   get_config
+  set_config
   default_config
 };
 
-use JSON;
 
 our $PREFIX = '/etc';
 our $IDENT  = 'authentication_milter';
 my  $CONFIG;
 
+
 sub default_config {
     my $config = {
         'debug'                           => 0,
@@ -45,9 +52,12 @@ sub default_config {
         'tempfail_on_error_authenticated' => '0',
         'tempfail_on_error_local'         => '0',
         'tempfail_on_error_trusted'       => '0',
+        'milter_quarantine'               => '0',
+        'ip_map'                          => {},
         'handlers'                        => {}
     };
 
+    require Mail::Milter::Authentication;
     my $installed_handlers = Mail::Milter::Authentication::get_installed_handlers();
     foreach my $handler ( @$installed_handlers ) {
         my $handler_module = 'Mail::Milter::Authentication::Handler::' . $handler;
@@ -64,6 +74,20 @@ sub default_config {
 
 }
 
+
+sub set_config {
+    my ( $config ) = @_;
+
+    my @load_handlers = keys %{ $config->{'handlers'} };
+    @load_handlers = grep { ! /^\!/ } @load_handlers;
+    $config->{'load_handlers'} = \@load_handlers;
+
+    my $protocol = $config->{'protocol'} || 'milter';
+    $config->{'protocol'} = $protocol;
+    $CONFIG = $config;
+}
+
+
 sub load_file {
     my ( $file ) = @_;
 
@@ -80,21 +104,49 @@ sub load_file {
         $text = join( q{}, @t );
     }
 
-    my $json = JSON->new();
-    $json->relaxed(1);
-    my $data = $json->decode($text)
-      || die "Error parsing config file $file";
+    my $data;
+
+    if ( $file =~ /\.toml$/ ) {
+        $data = TOML::from_toml($text)
+          || die "Error parsing config file $file";
+    }
+    else {
+        my $json = JSON->new();
+        $json->relaxed(1);
+        $data = $json->decode($text)
+          || die "Error parsing config file $file";
+    }
 
     return $data;
 }
 
+
+sub process_config {
+
+    if ( exists( $CONFIG->{ '_external_callback_processor' } ) ) {
+        if ( $CONFIG->{ '_external_callback_processor' }->can( 'process_config' ) ) {
+            $CONFIG->{ '_external_callback_processor' }->process_config( $CONFIG );
+        }
+    }
+
+    return $CONFIG;
+}
+
+
 sub get_config {
 
-    return $CONFIG if $CONFIG;
+    if ( $CONFIG ) {
+        return process_config();
+    }
 
-    my $file = $PREFIX . '/authentication_milter.json';
-
-    my $config = load_file( $file );
+    my $file = $PREFIX . '/authentication_milter';
+    my $config;
+    if ( -e $file . '.toml' ) {
+        $config = load_file( $file . '.toml' );
+    }
+    else {
+        $config = load_file( $file . '.json' );
+    }
 
     my $folder = $PREFIX . '/authentication_milter.d';
     if ( -d $folder ) {
@@ -102,12 +154,12 @@ sub get_config {
         opendir $dh, $folder;
         my @config_files =
             sort
-            grep { $_ =~ /\.json$/ }
+            grep { $_ =~ /\.(json|toml)$/ }
             grep { not $_ =~ /^\./ }
             readdir($dh);
         closedir $dh;
         foreach my $file ( @config_files ) {
-            $file =~ /(^.*)\.json$/;
+            $file =~ /(^.*)\.(json|toml)$/;
             my $handler = $1;
             ## ToDo Consider what to do if config already exists in .json config
             $config->{'handlers'}->{$handler} = load_file( join( '/', $folder, $file ) );
@@ -121,9 +173,26 @@ sub get_config {
     my $protocol = $config->{'protocol'} || 'milter';
     $config->{'protocol'} = $protocol;
 
+    # Have we specified an external callback processor?
+    if ( exists( $config->{ 'external_callback_processor' } ) ) {
+        # Try and load the handler
+        my $handler = $config->{ 'external_callback_processor' };
+        if ( ! is_loaded ( $handler ) ) {
+            eval {
+                no strict 'refs'; ## no critic;
+                load $handler;
+                $config->{ '_external_callback_processor' } = $handler->new();
+            };
+            if ( my $error = $@ ) {
+                delete $config->{ 'external_callback_processor' };
+                warn "Error loading external callback processor module: $error";
+            }
+        }
+    }
+
     $CONFIG = $config;
 
-    return $config;
+    return process_config();
 
 }
 
@@ -131,13 +200,17 @@ sub get_config {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Mail::Milter::Authentication::Config - Load config files for Authentication Milter
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-Load in configuration data.
+version 2.20200329.1
 
 =head1 SYNOPSIS
 
@@ -147,36 +220,59 @@ config to the caller.
 If the $Mail::Milter::Authentication::Config::PREFIX variable is set then the config file
 will be read from the supplied directory rather than /etc/
 
+=head1 DESCRIPTION
+
+Load in configuration data.
+
 =head1 FUNCTIONS
 
-=over
-
-=item I<default_config()>
+=head2 I<default_config()>
 
 Return a default configuration including defaults from handler modules.
 
-=item I<load_file()>
+This is not the default config used by the system if no config is present, rather it is the config
+which is presented to the user as an example default config when using the help feature.
+
+=head2 I<set_config( $config )>
+
+Set the config hashref, primarily used for testing.
+
+=head2 I<load_file( $file )>
 
 Internal function used to load the config from /etc/authentication_milter.json
 
-=item I<get_config()>
+=head2 I<process_config()>
+
+Process the loaded config with the callback if required.
+
+This is the name of a Module to load, the process_config method of the instantiated object
+will be called with $config as the argument.g
+
+    package ConfigProcessor;
+
+    sub new {
+        ...
+    }
+
+    sub process_config {
+        my ( $self, $config ) = @_;
+    }
+
+    1;
+
+=head2 I<get_config()>
 
 Return the config hashref, load from file(s) if required.
 
-=back
+=head1 AUTHOR
 
-=head1 DEPENDENCIES
+Marc Bradshaw <marc@marcbradshaw.net>
 
-  JSON
+=head1 COPYRIGHT AND LICENSE
 
-=head1 AUTHORS
+This software is copyright (c) 2020 by Marc Bradshaw.
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
