@@ -1,11 +1,30 @@
 package Mail::Milter::Authentication::Constants;
+# ABSTRACT: Define and export useful constants
+use 5.20.0;
 use strict;
 use warnings;
+##use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Common constants
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Exporter';
-use version; our $VERSION = version->declare('v1.1.7');
+
 
 ## no critic [Modules::ProhibitAutomaticExportation]
 
+# Syslog Constants
+
+use constant LOG_DEBUG   => 7;
+use constant LOG_INFO    => 6;
+use constant LOG_NOTICE  => 5;
+use constant LOG_WARNING => 4;
+use constant LOG_ERR     => 3;
+use constant LOG_CRIT    => 2;
+use constant LOG_ALERT   => 1;
+use constant LOG_EMERG   => 0;
+use constant LOG_MAIL    => 16;
+
+# Sendmail Constants
+
 use constant SMFIA_UNKNOWN      => 'U';
 use constant SMFIA_UNIX         => 'L';
 use constant SMFIA_INET         => '4';
@@ -71,6 +90,15 @@ use constant SMFI_V2_ACTS       => SMFI_V1_ACTS|SMFIF_CHGHDRS;
 use constant SMFI_CURR_ACTS     => SMFI_V2_ACTS;
 
 our @EXPORT = qw(
+    LOG_DEBUG
+    LOG_INFO
+    LOG_NOTICE
+    LOG_WARNING
+    LOG_ERR
+    LOG_CRIT
+    LOG_ALERT
+    LOG_EMERG
+    LOG_MAIL
     SMFIA_UNKNOWN
     SMFIA_UNIX
     SMFIA_INET
@@ -132,40 +160,39 @@ our @EXPORT = qw(
 our @EXPORT_OK = ( @EXPORT );
 our %EXPORT_TAGS = ( 'all' => [ @EXPORT_OK ] );
 
-
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Mail::Milter::Authentication::Constants - Constant declarations
+Mail::Milter::Authentication::Constants - Define and export useful constants
+
+=head1 VERSION
+
+version 2.20200329.1
+
+=head1 SYNOPSIS
+
+Constants defined here are used in the sendmail milter protocol
 
 =head1 DESCRIPTION
 
 Exports useful constants.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-Constants defined here are used in the sendmail milter protocol.
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 FUNCTIONS
+=head1 COPYRIGHT AND LICENSE
 
-none
-
-=head1 DEPENDENCIES
-
-none
-
-=head1 AUTHORS
-
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
