@@ -1,11 +1,12 @@
 package Mail::Milter::Authentication::Handler::AlignedFrom;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for Address alignment
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
 use Net::DNS;
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -31,42 +32,57 @@ sub envfrom_callback {
 
     # Defaults
     $self->{ 'from_header_count' } = 0;
+    $self->{ 'envfrom_count' } = 0;
     $self->{ 'smtp_address' } = q{};
     $self->{ 'smtp_domain' } = q{};
     $self->{ 'header_address' } = q{};
     $self->{ 'header_domain' } = q{};
 
-    my $email = $self->get_address_from( $env_from );
-    return if ! $email;
-    $self->{ 'smtp_address'} = lc $email;
-    $self->{ 'smtp_domain'} = lc $self->get_domain_from( $email );
-
-    return;
+    my $emails = $self->get_addresses_from( $env_from );
+    foreach my $email ( @$emails ) {
+        next if ! $email;
+        $self->{ 'envfrom_count' } = $self->{ 'envfrom_count' } + 1;
+        # More than 1 here? we set to error in eom callback.!
+        $self->{ 'smtp_address'} = lc $email;
+        $self->{ 'smtp_domain'} = lc $self->get_domain_from( $email );
+    }
 }
 
 sub header_callback {
     my ( $self, $header, $value ) = @_;
 
-    return if $header ne 'From';
+    return if lc $header ne 'from';
 
-    $self->{ 'from_header_count' } = $self->{ 'from_header_count' } + 1;
+    my $emails = $self->get_addresses_from( $value );
 
-    my $email = $self->get_address_from( $value );
-    return if ! $email;
-    $self->{ 'header_address'} = lc $email;
-    $self->{ 'header_domain'} = lc $self->get_domain_from( $email );
+    my $found_domains = {};
 
-    return;
+
+    foreach my $email ( @$emails ) {
+        next if ! $email;
+        $self->{ 'header_address'} = lc $email;
+        my $domain = lc $self->get_domain_from( $email );
+        $self->{ 'header_domain'} = $domain;
+        $found_domains->{ $domain } = $1;
+    }
+
+    # We don't consider finding 2 addresses at the same domain in a header to be 2 separate entries
+    # for alignment checking, only count them as one.
+    foreach my $domain ( sort keys %$found_domains ) {
+        $self->{ 'from_header_count' } = $self->{ 'from_header_count' } + 1;
+        # If there are more than 1 then the result will be set to error in the eom callback
+        # Multiple from headers should always set the result to error.
+    }
 }
 
 sub close_callback {
     my ( $self ) = @_;
+    delete $self->{ 'envfrom_count' };
     delete $self->{ 'from_header_count' };
     delete $self->{ 'header_address' };
     delete $self->{ 'header_domain' };
     delete $self->{ 'smtp_address' };
     delete $self->{ 'smtp_domain' };
-    return;
 }
 
 # error = multiple from headers present
@@ -81,29 +97,41 @@ sub eom_callback {
     my ( $self ) = @_;
 
     my $result;
+    my $comment;
 
     if ( $self->{ 'from_header_count' } > 1 ) {
         $result = 'error';
+        $comment = 'Multiple addresses in header';
+    }
+
+    elsif ( $self->{ 'envfrom_count' } > 1 ) {
+        $result = 'error';
+        $comment = 'Multiple addresses in envelope';
     }
 
     elsif ( ( ! $self->{ 'smtp_domain' } ) && ( ! $self->{ 'header_domain' } ) ) {
         $result = 'null';
+        $comment = 'No domains found';
     }
 
     elsif ( ! $self->{ 'smtp_domain' } ) {
         $result = 'null_smtp';
+        $comment = 'No envelope domain';
     }
 
     elsif ( ! $self->{ 'header_domain' } ) {
         $result = 'null_header';
+        $comment = 'No header domain';
     }
 
     elsif ( $self->{ 'smtp_address' } eq $self->{ 'header_address' } ) {
         $result = 'pass';
+        $comment = 'Address match';
     }
 
     elsif ( $self->{ 'smtp_domain' } eq $self->{ 'header_domain' } ) {
         $result = 'domain_pass';
+        $comment = 'Domain match';
     }
 
     else {
@@ -113,10 +141,13 @@ sub eom_callback {
             my $dmarc_handler = $self->get_handler('DMARC');
             my $dmarc_object = $dmarc_handler->get_dmarc_object();
             my $org_smtp_domain   = eval{ $dmarc_object->get_organizational_domain( $self->{ 'smtp_domain' } ); };
+            $self->handle_exception( $@ );
             my $org_header_domain = eval{ $dmarc_object->get_organizational_domain( $self->{ 'header_domain' } ); };
+            $self->handle_exception( $@ );
 
             if ( $org_smtp_domain eq $org_header_domain ) {
                 $result = 'orgdomain_pass';
+                $comment = 'Domain org match';
             }
 
             else {
@@ -131,22 +162,31 @@ sub eom_callback {
 
     }
 
-
     $self->dbgout( 'AlignedFrom', $result, LOG_DEBUG );
-    $self->add_auth_header( $self->format_header_entry( 'x-aligned-from', $result ) );
+    my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-aligned-from' )->safe_set_value( $result );
+    if ( $comment ) {
+      $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $comment ) );
+    }
+    $self->add_auth_header( $header );
 
     $self->metric_count( 'alignedfrom_total', { 'result' => $result } );
-
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - AlignedFrom Module
+Mail::Milter::Authentication::Handler::AlignedFrom - Handler class for Address alignment
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -156,17 +196,15 @@ Check that Mail From and Header From addresses are in alignment.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
