@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Handler::Auth;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for SMTP Auth
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -29,7 +30,6 @@ sub pre_loop_setup {
     if ( $protocol ne 'milter' ) {
         warn 'The Auth handler only works with the milter protocol';
     }
-    return;
 }
 
 sub get_auth_name {
@@ -41,7 +41,6 @@ sub get_auth_name {
 sub connect_callback {
     my ( $self, $hostname, $ip ) = @_;
     $self->{'is_authenticated'} = 0;
-    return;
 }
 
 sub envfrom_callback {
@@ -56,24 +55,31 @@ sub envfrom_callback {
         $top_handler->{'auth_headers'}   = [];
         $self->{'is_authenticated'}       = 1;
         $self->metric_count( 'authenticated_connect_total' );
-        $self->add_auth_header('auth=pass');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'auth' )->safe_set_value( 'pass' );
+        $self->add_auth_header( $header );
     }
-    return;
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'is_authenticated'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - Auth Module
+Mail::Milter::Authentication::Handler::Auth - Handler class for SMTP Auth
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -83,17 +89,15 @@ Module which identifies email that was sent via an authenticated connection.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
