@@ -1,25 +1,25 @@
 package Mail::Milter::Authentication::Handler::DKIM;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for DKIM
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Data::Dumper;
-use English qw{ -no_match_vars };
-use Sys::Syslog qw{:standard :macros};
-
-use Mail::DKIM;
-use Mail::DKIM::Verifier;
+use Mail::DKIM 0.39;
 use Mail::DKIM::DNS;
+use Mail::DKIM::KeyValueList;
+use Mail::DKIM::Verifier 0.39;
 
 sub default_config {
     return {
         'hide_none'         => 0,
         'hide_domainkeys'   => 0,
         'check_adsp'        => 1,
-        'show default_adsp' => 0,
+        'show_default_adsp' => 0,
         'adsp_hide_none'    => 0,
         'extra_properties'  => 0,
+        'no_strict'         => 0,
     };
 }
 
@@ -45,7 +45,6 @@ sub envfrom_callback {
     $self->{'valid_domains'} = {};
     $self->{'carry'}         = q{};
     $self->destroy_object('dkim');
-    return;
 }
 
 sub show_domainkeys {
@@ -57,21 +56,37 @@ sub show_domainkeys {
 }
 
 sub header_callback {
-    my ( $self, $header, $value ) = @_;
+    my ( $self, $header, $value, $original ) = @_;
     return if ( $self->{'failmode'} );
     my $EOL        = "\015\012";
-    my $dkim_chunk = $header . ': ' . $value . $EOL;
+    my $dkim_chunk = $original . $EOL;
     $dkim_chunk =~ s/\015?\012/$EOL/g;
     push @{$self->{'headers'}} , $dkim_chunk;
 
     if ( lc($header) eq 'dkim-signature' ) {
         $self->{'has_dkim'} = 1;
+
+        my $parsed = eval{ Mail::DKIM::KeyValueList->parse( $value ) };
+        $self->handle_exception( $@ );
+        if ( $parsed ) {
+            my $domain = $parsed->get_tag('d');
+            my $selector = $parsed->get_tag('s');
+            if ( $selector && $domain ) {
+                my $resolver = $self->get_object('resolver');
+                my $lookup = $selector.'._domainkey.'.$domain;
+                eval{ $resolver->bgsend( $lookup, 'TXT' ) };
+                $self->handle_exception( $@ );
+                $self->dbgout( 'DNSEarlyLookup', "$lookup TXT", LOG_DEBUG );
+                $lookup = '_adsp._domainkey.'.$domain;
+                eval{ $resolver->bgsend( $lookup, 'TXT' ) };
+                $self->handle_exception( $@ );
+                $self->dbgout( 'DNSEarlyLookup', "$lookup TXT", LOG_DEBUG );
+            }
+        }
     }
     if ( lc($header) eq 'domainkey-signature' ) {
         $self->{'has_dkim'} = 1 if $self->show_domainkeys();
     }
-
-    return;
 }
 
 sub eoh_callback {
@@ -83,9 +98,9 @@ sub eoh_callback {
         $self->metric_count( 'dkim_total', { 'result' => 'none' } );
         $self->dbgout( 'DKIMResult', 'No DKIM headers', LOG_INFO );
         if ( !( $config->{'hide_none'} ) ) {
-            $self->add_auth_header(
-                $self->format_header_entry( 'dkim', 'none' )
-                . ' (no signatures found)' );
+            my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( 'none' );
+            $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'no signatures found' ) );
+            $self->add_auth_header( $header );
         }
         delete $self->{'headers'};
     }
@@ -93,7 +108,11 @@ sub eoh_callback {
 
         my $dkim;
         eval {
-            $dkim = Mail::DKIM::Verifier->new();
+            my $UseStrict = 1;
+            if ( $config->{ 'no_strict' } ) {
+                $UseStrict = 0;
+            }
+            $dkim = Mail::DKIM::Verifier->new( 'Strict' => $UseStrict );
             # The following requires Mail::DKIM > 0.4
             if ( $Mail::DKIM::VERSION >= 0.4 ) {
                 my $resolver = $self->get_object('resolver');
@@ -102,10 +121,11 @@ sub eoh_callback {
             $self->set_object('dkim', $dkim, 1);
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'DKIM Setup Error ' . $error );
+            $self->{'failmode'} = 1;
             $self->_check_error( $error );
             $self->metric_count( 'dkim_total', { 'result' => 'error' } );
-            $self->{'failmode'} = 1;
             delete $self->{'headers'};
             return;
         }
@@ -117,18 +137,17 @@ sub eoh_callback {
             );
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'DKIM Headers Error ' . $error );
+            $self->{'failmode'} = 1;
             $self->_check_error( $error );
             $self->metric_count( 'dkim_total', { 'result' => 'error' } );
-            $self->{'failmode'} = 1;
         }
 
         delete $self->{'headers'};
     }
 
     $self->{'carry'} = q{};
-
-    return;
 }
 
 sub body_callback {
@@ -158,12 +177,12 @@ sub body_callback {
         $dkim->PRINT( $dkim_chunk );
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'DKIM Body Error ' . $error );
+        $self->{'failmode'} = 1;
         $self->_check_error( $error );
         $self->metric_count( 'dkim_total', { 'result' => 'error' } );
-        $self->{'failmode'} = 1;
     }
-    return;
 }
 
 sub eom_callback {
@@ -179,6 +198,7 @@ sub eom_callback {
     eval {
         $dkim->PRINT( $self->{'carry'} );
         $dkim->CLOSE();
+        $self->check_timeout();
 
         my $dkim_result        = $dkim->result;
         my $dkim_result_detail = $dkim->result_detail;
@@ -189,9 +209,9 @@ sub eom_callback {
 
         if ( !$dkim->signatures() ) {
             if ( !( $config->{'hide_none'} && $dkim_result eq 'none' ) ) {
-                $self->add_auth_header(
-                    $self->format_header_entry( 'dkim', $dkim_result )
-                      . ' (no signatures found)' );
+                my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( $dkim_result );
+                $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'no signatures found' ) );
+                $self->add_auth_header( $header );
             }
         }
         foreach my $signature ( $dkim->signatures() ) {
@@ -225,7 +245,7 @@ sub eom_callback {
             if ( $signature_result ne 'pass' and $signature_result ne 'none' ) {
                 $signature_result_detail =~ /$signature_result \((.*)\)/;
                 if ( $1 ) {
-                    $result_comment = $1 . '; ';
+                    $result_comment = $1 . ', ';
                 }
             }
             if (
@@ -260,58 +280,34 @@ sub eom_callback {
 
                 if ( $type eq 'domainkeys' ) {
                     if ( $self->show_domainkeys() ) {
-                        my $header = join(
-                            q{ },
-                            $self->format_header_entry( $type, $signature_result ),
-                            '('
-                              . $self->format_header_comment(
-                                  $result_comment
-                                  . $key_data
-                                )
-                              . ')',
-                            $self->format_header_entry( 'header.d', $signature->domain() ),
-                            $self->format_header_entry( 'header.b', substr( $signature->data(), 0, 8 ) ),
-                        );
+                        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( $type )->safe_set_value( $signature_result );
+                        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $result_comment . $key_data ) );
+                        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.d' )->safe_set_value( $signature->domain() ) );
+                        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.b' )->safe_set_value( substr( $signature->data(), 0, 8 ) ) );
                         if ( $config->{'extra_properties'} ) {
-                            $header = join(
-                                q{ },
-                                $header,
-                                $self->format_header_entry( 'x-bits', $key_size ),
-                                $self->format_header_entry( 'x-keytype', $key_type ),
-                                $self->format_header_entry( 'x-algorithm', $hash_algorithm ),
-                                $self->format_header_entry( 'x-selector', $selector ),
-                            );
+                            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-bits' )->safe_set_value( $key_size ) );
+                            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-keytype' )->safe_set_value( $key_type ) );
+                            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-algorithm' )->safe_set_value( $hash_algorithm ) );
+                            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-selector' )->safe_set_value( $selector ) );
                         }
                         $self->add_auth_header($header);
-                        }
+                    }
                 }
                 else {
-                    my $header = join(
-                        q{ },
-                        $self->format_header_entry( $type, $signature_result ),
-                        '('
-                          . $self->format_header_comment(
-                            $result_comment
-                            . $key_data
-                          )
-                          . ')',
-                        $self->format_header_entry( 'header.d', $signature->domain() ),
-                        $self->format_header_entry( 'header.i', $signature->identity() ),
-                        $self->format_header_entry( 'header.b', substr( $signature->data(), 0, 8 ) ),
-                    );
+                    my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( $type )->safe_set_value( $signature_result );
+                    $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $result_comment . $key_data ) );
+                    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.d' )->safe_set_value( $signature->domain() ) );
+                    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.i' )->safe_set_value( $signature->identity() ) );
+                    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.b' )->safe_set_value( substr( $signature->data(), 0, 8 ) ) );
+                    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.a' )->safe_set_value( $key_type . '-' . $hash_algorithm ) );
+                    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.s' )->safe_set_value( $selector ) );
                     if ( $config->{'extra_properties'} ) {
-                        $header = join(
-                            q{ },
-                            $header,
-                            $self->format_header_entry( 'x-bits', $key_size ),
-                            $self->format_header_entry( 'x-keytype', $key_type ),
-                            $self->format_header_entry( 'x-algorithm', $hash_algorithm ),
-                            $self->format_header_entry( 'x-selector', $selector ),
-                        );
+                        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-bits' )->safe_set_value( $key_size ) );
                     }
                     $self->add_auth_header($header);
                 }
             }
+            $self->check_timeout();
         }
 
         # the alleged author of the email may specify how to handle email
@@ -351,29 +347,28 @@ sub eom_callback {
 
                 if ( ! ( $config->{'adsp_hide_none'} && $result eq 'none' ) ) {
                     if ( ( ! $default ) or $config->{'show_default_adsp'} ) {
-                        my $comment = '('
-                          . $self->format_header_comment( ( $default ? 'default ' : q{} )
-                            . "$name policy"
-                            . ( $location ? " from $location" : q{} )
-#                            . ( $string   ? "; $string"       : q{} )
-                          )
-                          . ')';
-
-                        my $header = join( q{ },
-                            $self->format_header_entry( $type, $result ), $comment, );
+                        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( $type )->safe_set_value( $result );
+                        my $comment = ( $default ? 'default ' : q{} )
+                                    . "$name policy"
+                                    . ( $location ? " from $location" : q{} )
+#                                   . ( $string   ? ", $string"       : q{} )
+                        ;
+                        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $comment ) );
                         $self->add_auth_header( $header );
                     }
                 }
+                $self->check_timeout();
             }
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
 
         # Also in DMARC module
         $self->log_error( 'DKIM EOM Error ' . $error );
+        $self->{'failmode'} = 1;
         $self->_check_error( $error );
         $self->metric_count( 'dkim_total', { 'result' => 'error' } );
-        $self->{'failmode'} = 1;
         return;
     }
 }
@@ -387,7 +382,6 @@ sub close_callback {
     delete $self->{'has_dkim'};
     delete $self->{'valid_domains'};
     $self->destroy_object('dkim');
-    return;
 }
 
 sub _check_error {
@@ -396,11 +390,15 @@ sub _check_error {
             or $error =~ /^DNS query timeout/
     ){
         $self->log_error( 'Temp DKIM Error - ' . $error );
-        $self->add_auth_header('dkim=temperror (dns timeout)');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( 'temperror' );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'dns timeout' ) );
+        $self->add_auth_header( $header );
     }
     elsif ( $error =~ /^DNS error: SERVFAIL/ ){
         $self->log_error( 'Temp DKIM Error - ' . $error );
-        $self->add_auth_header('dkim=temperror (dns servfail)');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( 'temperror' );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'dns servfail' ) );
+        $self->add_auth_header( $header );
     }
     elsif ( $error =~ /^no domain to fetch policy for$/
             or $error =~ /^policy syntax error$/
@@ -408,26 +406,35 @@ sub _check_error {
             or $error =~ /^invalid name /
     ){
         $self->log_error( 'Perm DKIM Error - ' . $error );
-        $self->add_auth_header('dkim=permerror (syntax or domain error)');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( 'permerror' );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'syntax or domain error' ) );
+        $self->add_auth_header( $header );
     }
     else {
-        $self->log_error( 'Unexpected DKIM Error - ' . $error );
-        $self->add_auth_header('dkim=temperror');
+        $self->exit_on_close( 'Unexpected DKIM Error - ' . $error );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dkim' )->safe_set_value( 'temperror' );
+        $self->add_auth_header( $header );
         # Fill these in as they occur, but for unknowns err on the side of caution
         # and tempfail/exit
-        $self->exit_on_close();
         $self->tempfail_on_error();
     }
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - DKIM Module
+Mail::Milter::Authentication::Handler::DKIM - Handler class for DKIM
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -442,19 +449,18 @@ Module for validation of DKIM and DomainKeys signatures, and application of ADSP
             "show_default_adsp" : 0,                    | Show the default ADSP result
             "adsp_hide_none"    : 0,                    | Hide auth ADSP if the result is 'none'
             "extra_properties"  : 0                     | Add extra properties (not to rfc) relating to key and selector
+            "no_strict"         : 0,                    | Ignore rfc 8301 security considerations (not recommended
         },
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
