@@ -1,12 +1,13 @@
 package Mail::Milter::Authentication::Handler::IPRev;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for IPRev
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
 use Net::DNS;
 use Net::IP;
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -41,7 +42,6 @@ sub _dns_error {
           . ' with '
           . $error );
     }
-    return;
 }
 
 sub connect_requires {
@@ -83,7 +83,7 @@ sub connect_callback {
 
     if ( $resolver->errorstring() ) {
         $self->_dns_error( 'PTR', $ip_address, $resolver->errorstring );
-        push @error_list, 'Error ' . $resolver->errorstring() . " looking up $ip_address PTR";
+        push @error_list, 'Error ' . $resolver->errorstring() . " looking up $ip_address PTR" if ( $resolver->errorstring() ne 'unknown error or no error' );
     }
 
     foreach my $cname_host ( @cname_hosts ) {
@@ -102,7 +102,7 @@ sub connect_callback {
         }
         if ( $resolver->errorstring() ) {
             $self->_dns_error( 'PTR', $cname_host, $resolver->errorstring );
-            push @error_list, 'Error ' . $resolver->errorstring() . " looking up $cname_host PTR";
+            push @error_list, 'Error ' . $resolver->errorstring() . " looking up $cname_host PTR" if ( $resolver->errorstring() ne 'unknown error or no error' );
         }
         last; # Because multiple CNAMES for a given record is also busted
     }
@@ -176,10 +176,10 @@ sub connect_callback {
             }
             if ( ! @$ip_list ) {
                 foreach my $error ( @$errors4 ) {
-                    push @error_list, "Error $error looking up $domain A";
+                    push @error_list, "Error $error looking up $domain A" if ( $resolver->errorstring() ne 'unknown error or no error' );
                 }
                 foreach my $error ( @$errors6 ) {
-                    push @error_list, "Error $error looking up $domain AAAA";
+                    push @error_list, "Error $error looking up $domain AAAA" if ( $resolver->errorstring() ne 'unknown error or no error' );
                 }
             }
 
@@ -205,10 +205,9 @@ sub connect_callback {
         # Failed to match IP against looked up domains
         my $comment = join( ',', @error_list );
         $self->dbgout( 'IPRevCheck', "fail - $comment", LOG_DEBUG );
-        my $header =
-            $self->format_header_entry( 'iprev',        'fail' ) . ' '
-          . $self->format_header_entry( 'policy.iprev', $ip_address ) . ' ' . '('
-          . $self->format_header_comment($comment) . ')';
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'iprev' )->safe_set_value( 'fail' );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.remote-ip' )->safe_set_value( $ip_address ) );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $comment ) );
         $self->add_c_auth_header($header);
         $self->metric_count( 'iprev_total', { 'result' => 'fail'} );
     }
@@ -217,15 +216,12 @@ sub connect_callback {
         my $comment = join( ',', @match_list );
         $self->{'verified_ptr'} = $comment;
         $self->dbgout( 'IPRevCheck', "pass - $comment", LOG_DEBUG );
-        my $header =
-            $self->format_header_entry( 'iprev',        'pass' ) . ' '
-          . $self->format_header_entry( 'policy.iprev', $ip_address ) . ' ' . '('
-          . $self->format_header_comment($comment) . ')';
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'iprev' )->safe_set_value( 'pass' );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.remote-ip' )->safe_set_value( $ip_address ) );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $comment ) );
         $self->add_c_auth_header($header);
         $self->metric_count( 'iprev_total', { 'result' => 'pass'} );
     }
-
-    return;
 }
 
 sub _address_for_domain {
@@ -267,16 +263,23 @@ sub _address_for_domain {
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'verified_ptr'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - IPRev Module
+Mail::Milter::Authentication::Handler::IPRev - Handler class for IPRev
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -286,17 +289,15 @@ Check reverse IP lookups.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
