@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Handler::LocalIP;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for Local IP Connections
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -20,10 +21,6 @@ sub grafana_rows {
 sub is_local_ip_address {
     my ( $self, $ip ) = @_;
     my $ip_address = $ip->ip();
-    if ( ! $ip ) {
-        $self->dbgout( 'IPAddress', "Address $ip_address detected as invalid", LOG_DEBUG );
-        return 0;
-    }
     my $ip_type  = $ip->iptype();
     my $type_map = {
         'PRIVATE'              => 1,
@@ -62,26 +59,33 @@ sub connect_callback {
     $self->{'is_local_ip_address'} = 0;
     if ( $self->is_local_ip_address($ip) ) {
         $self->dbgout( 'LocalIP', 'pass', LOG_DEBUG );
-        $self->add_c_auth_header('x-local-ip=pass');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-local-ip' )->safe_set_value( 'pass' );
+        $self->add_c_auth_header( $header );
         $self->{'is_local_ip_address'} = 1;
         $self->metric_count( 'localip_connect_total' );
     }
-    return;
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'is_local_ip_address'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - LocalIP Module
+Mail::Milter::Authentication::Handler::LocalIP - Handler class for Local IP Connections
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -91,17 +95,15 @@ Detect a Local IP address and act accordingly.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
