@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Handler::PTR;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for PTR checking
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -50,32 +51,30 @@ sub helo_callback {
         }
     }
 
-    if ( $found_match ) {
-        $self->dbgout( 'PTRMatch', 'pass', LOG_DEBUG );
-        $self->add_c_auth_header(
-                $self->format_header_entry( 'x-ptr',        'pass' ) . q{ }
-              . $self->format_header_entry( 'x-ptr-helo',   $helo_host ) . q{ }
-              . $self->format_header_entry( 'x-ptr-lookup', $domains ) );
-        $self->metric_count( 'ptr_total', { 'result' => 'pass'} );
-    }
-    else {
-        $self->dbgout( 'PTRMatch', 'fail', LOG_DEBUG );
-        $self->add_c_auth_header(
-                $self->format_header_entry( 'x-ptr',        'fail' ) . q{ }
-              . $self->format_header_entry( 'x-ptr-helo',   $helo_host ) . q{ }
-              . $self->format_header_entry( 'x-ptr-lookup', $domains ) );
-        $self->metric_count( 'ptr_total', { 'result' => 'fail'} );
-    }
-    return;
+    my $result = $found_match ? 'pass' : 'fail';
+    $self->dbgout( 'PTRMatch', $result, LOG_DEBUG );
+    my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-ptr' )->safe_set_value( $result );
+    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.helo' )->safe_set_value( $helo_host ) );
+    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.ptr' )->safe_set_value( $domains ) );
+    $self->add_c_auth_header( $header );
+    $self->metric_count( 'ptr_total', { 'result' => $result} );
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - PTR Module
+Mail::Milter::Authentication::Handler::PTR - Handler class for PTR checking
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -87,17 +86,15 @@ This handler requires the IPRev handler to be installed and active.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
