@@ -1,11 +1,11 @@
 package Mail::Milter::Authentication::Handler::SPF;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for SPF
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
-
 use Mail::SPF;
 
 sub default_config {
@@ -39,6 +39,7 @@ sub setup_callback {
             );
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'SPF Object Setup Error ' . $error );
         }
         $thischild->{'object'}->{$name} = {
@@ -46,7 +47,6 @@ sub setup_callback {
             'destroy' => 0,
         };
     });
-    return;
 }
 
 sub register_metrics {
@@ -67,7 +67,6 @@ sub helo_callback {
     my ( $self, $helo_host ) = @_;
     $self->{'failmode'} = 0;
     $self->{'helo_name'} = $helo_host;
-    return;
 }
 
 sub envfrom_callback {
@@ -84,7 +83,8 @@ sub envfrom_callback {
     if ( ! $spf_server ) {
         $self->log_error( 'SPF Setup Error' );
         $self->metric_count( 'spf_total', { 'result' => 'error' } );
-        $self->add_auth_header('spf=temperror');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'spf' )->safe_set_value( 'temperror' );
+        $self->add_auth_header($header);
         return;
     }
 
@@ -120,19 +120,26 @@ sub envfrom_callback {
         );
 
         my $spf_result = $spf_server->process($spf_request);
+        my $spf_results = $self->get_object('spf_results');
+        $spf_results = [] if ! $spf_results;
+        push @$spf_results, $spf_result;
+        $self->set_object('spf_results',$spf_results,1);
 
         my $result_code = $spf_result->code();
 
         # Best Guess SPF based on org domain
         # ToDo report this in both metrics and AR header
-        if ( $spf_result eq 'none' ) {
+        my $auth_domain;
+        if ( $result_code eq 'none' ) {
             if ( $config->{'best_guess'} ) {
                 if ( $self->is_handler_loaded( 'DMARC' ) ) {
                     my $dmarc_handler = $self->get_handler('DMARC');
                     my $dmarc_object = $dmarc_handler->get_dmarc_object();
                     if ( $domain ) {
                         my $org_domain = eval{ $dmarc_object->get_organizational_domain( $domain ); };
-                        if ( $org_domain eq $domain ) {
+                        $self->handle_exception( $@ );
+                        if ( $org_domain ne $domain ) {
+                            $auth_domain = $org_domain;
                             $spf_request = Mail::SPF::Request->new(
                                 'versions'         => [1],
                                 'scope'            => $scope,
@@ -142,6 +149,10 @@ sub envfrom_callback {
                                 'helo_identity'    => $self->{'helo_name'},
                             );
                             $spf_result = $spf_server->process($spf_request);
+                            my $spf_results = $self->get_object('spf_results');
+                            $spf_results = [] if ! $spf_results;
+                            push @$spf_results, $spf_result;
+                            $self->set_object('spf_results',$spf_results,1);
                             $result_code = $spf_result->code();
                         }
                     }
@@ -151,13 +162,14 @@ sub envfrom_callback {
 
         $self->metric_count( 'spf_total', { 'result' => $result_code } );
 
-        my $auth_header = join( q{ },
-            $self->format_header_entry( 'spf',           $result_code ),
-            $self->format_header_entry( 'smtp.mailfrom', $self->get_address_from( $env_from ) ),
-            $self->format_header_entry( 'smtp.helo',     $self->{'helo_name'} ),
-        );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'spf' )->safe_set_value( $result_code );
+        if ( $auth_domain ) {
+            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.authdomain' )->safe_set_value( $auth_domain ) );
+        }
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.mailfrom' )->safe_set_value( $self->get_address_from( $env_from ) ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.helo' )->safe_set_value( $self->{ 'helo_name' } ) );
         if ( !( $config->{'hide_none'} && $result_code eq 'none' ) ) {
-            $self->add_auth_header($auth_header);
+            $self->add_auth_header($header);
         }
 
         # Set for DMARC
@@ -178,13 +190,13 @@ sub envfrom_callback {
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'SPF Error ' . $error );
-        $self->add_auth_header('spf=temperror');
-        $self->metric_count( 'spf_total', { 'result' => 'error' } );
         $self->{'failmode'} = 1;
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'spf' )->safe_set_value( 'temperror' );
+        $self->add_auth_header($header);
+        $self->metric_count( 'spf_total', { 'result' => 'error' } );
     }
-
-    return;
 }
 
 sub close_callback {
@@ -194,16 +206,24 @@ sub close_callback {
     delete $self->{'dmarc_result'};
     delete $self->{'failmode'};
     delete $self->{'helo_name'};
-    return;
+    $self->destroy_object('spf_results');
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - SPF Module
+Mail::Milter::Authentication::Handler::SPF - Handler class for SPF
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -219,17 +239,15 @@ Implements the SPF standard checks.
                                                         | if result is none.
         },
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
