@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Handler::Sanitize;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for Removing headers
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {
@@ -30,6 +31,8 @@ sub is_hostname_mine {
     my ( $self, $check_hostname ) = @_;
     my $config = $self->handler_config();
 
+    return 0 if ! defined $check_hostname;
+
     my $hostname = $self->get_my_hostname();
     my ($check_for) = $hostname =~ /^[^\.]+\.(.*)/;
 
@@ -55,26 +58,42 @@ sub is_hostname_mine {
 
 sub remove_auth_header {
     my ( $self, $index ) = @_;
-    $self->metric_count( 'sanitize_remove_total' );
+    $self->metric_count( 'sanitize_remove_total', {'header'=>'authentication-results'} );
     if ( !exists( $self->{'remove_auth_headers'} ) ) {
         $self->{'remove_auth_headers'} = [];
     }
     push @{ $self->{'remove_auth_headers'} }, $index;
-    return;
+}
+
+sub get_headers_to_remove {
+    my ( $self ) = @_;
+    my @headers = qw{ X-Disposition-Quarantine };
+    return \@headers;
 }
 
 sub envfrom_callback {
     my ( $self, $env_from ) = @_;
     delete $self->{'auth_result_header_index'};
     delete $self->{'remove_auth_headers'};
-    return;
+
+    my $headers = {};
+    foreach my $header ( sort @{ $self->get_headers_to_remove() } ) {
+        $headers->{ lc $header } = {
+            'index'  => 0,
+            'silent' => 1,
+        };
+    }
+    $self->{'header_hash'} = $headers;
 }
 
 sub header_callback {
     my ( $self, $header, $value ) = @_;
     my $config = $self->handler_config();
+
     return if ( $self->is_trusted_ip_address() );
     return if ( lc $config->{'remove_headers'} eq 'no' );
+
+    # Sanitize Authentication-Results headers
     if ( lc $header eq 'authentication-results' ) {
         if ( !exists $self->{'auth_result_header_index'} ) {
             $self->{'auth_result_header_index'} = 0;
@@ -97,36 +116,71 @@ sub header_callback {
             }
         }
     }
-    return;
+
+    # Sanitize other headers
+    foreach my $remove_header ( sort @{ $self->get_headers_to_remove() } ) {
+        next if ( lc $remove_header ne lc $header );
+        $self->{'header_hash'}->{ lc $header }->{'index'} = $self->{'header_hash'}->{ lc $header }->{'index'} + 1;
+        $self->metric_count( 'sanitize_remove_total', {'header'=> lc $header} );
+
+        if ( ! $self->{'header_hash'}->{ lc $header }->{'silent'} ) {
+            my $forged_header =
+              '(Received ' . $remove_header . ' header removed by '
+              . $self->get_my_hostname()
+              . ')' . "\n"
+              . '    '
+              . $value;
+            $self->append_header( 'X-Received-' . $remove_header,
+                $forged_header );
+        }
+    }
 }
 
 sub eom_callback {
     my ($self) = @_;
     my $config = $self->handler_config();
     return if ( lc $config->{'remove_headers'} eq 'no' );
+
     if ( exists( $self->{'remove_auth_headers'} ) ) {
         foreach my $index ( reverse @{ $self->{'remove_auth_headers'} } ) {
             $self->dbgout( 'RemoveAuthHeader', $index, LOG_DEBUG );
             $self->change_header( 'Authentication-Results', $index, q{} );
         }
     }
-    return;
+
+    foreach my $remove_header ( sort @{ $self->get_headers_to_remove() } ) {
+        my $max_index = $self->{'header_hash'}->{ lc $remove_header }->{'index'};
+        if ( $max_index ) {
+            for ( my $index = $max_index; $index > 0; $index-- ) {
+                $self->dbgout( 'RemoveHeader', "$remove_header $index", LOG_DEBUG );
+                $self->change_header( $remove_header, $index, q{} );
+            }
+        }
+    }
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'remove_auth_headers'};
     delete $self->{'auth_result_header_index'};
-    return;
+    delete $self->{'header_hash'};
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - Sanitize Module
+Mail::Milter::Authentication::Handler::Sanitize - Handler class for Removing headers
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -147,17 +201,15 @@ Remove unauthorized (forged) Authentication-Results headers from processed email
                                                         | Does not run for trusted IP address connections
         }
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
