@@ -1,14 +1,13 @@
 package Mail::Milter::Authentication::Handler::SenderID;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for SenderID
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
-
-use Mail::SPF;
-
 use Mail::Milter::Authentication::Handler::SPF;
+use Mail::SPF;
 
 sub default_config {
     return {
@@ -39,7 +38,6 @@ sub register_metrics {
 sub helo_callback {
     my ( $self, $helo_host ) = @_;
     $self->{'helo_name'} = $helo_host;
-    return;
 }
 
 sub envfrom_callback {
@@ -48,7 +46,6 @@ sub envfrom_callback {
     return if ( $self->is_trusted_ip_address() );
     return if ( $self->is_authenticated() );
     delete $self->{'from_header'};
-    return;
 }
 
 sub header_callback {
@@ -59,7 +56,6 @@ sub header_callback {
     if ( lc $header eq 'from' ) {
         $self->{'from_header'} = $value;
     }
-    return;
 }
 
 sub eoh_callback {
@@ -72,7 +68,8 @@ sub eoh_callback {
     my $spf_server = $self->get_object('spf_server');
     if ( ! $spf_server ) {
         $self->log_error( 'SenderID Setup Error' );
-        $self->add_auth_header('senderid=temperror');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'senderid' )->safe_set_value( 'temperror' );
+        $self->add_auth_header($header);
         $self->metric_count( 'senderid_total', { 'result' => 'error' } );
         return;
     }
@@ -83,7 +80,8 @@ sub eoh_callback {
 
     if ( ! $identity ) {
         $self->log_error( 'SENDERID Error No Identity' );
-        $self->add_auth_header('senderid=permerror');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'senderid' )->safe_set_value( 'permerror' );
+        $self->add_auth_header( $header );
         $self->metric_count( 'senderid_total', { 'result' => 'permerror' } );
         return;
     }
@@ -104,7 +102,7 @@ sub eoh_callback {
         $self->dbgout( 'SenderIdCode', $result_code, LOG_INFO );
 
         if ( ! ( $config->{'hide_none'} && $result_code eq 'none' ) ) {
-            my $auth_header = $self->format_header_entry( 'senderid', $result_code );
+            my $auth_header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'senderid' )->safe_set_value( $result_code );
             $self->add_auth_header( $auth_header );
 #my $result_local  = $spf_result->local_explanation;
 #my $result_auth   = $spf_result->can( 'authority_explanation' ) ? $spf_result->authority_explanation() : '';
@@ -115,28 +113,36 @@ sub eoh_callback {
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'SENDERID Error ' . $error );
         $self->metric_count( 'senderid_total', { 'result' => 'error' } );
-        $self->add_auth_header('senderid=temperror');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'senderid' )->safe_set_value( 'temperror' );
+        $self->add_auth_header($header);
         return;
     }
-    return;
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'from_header'};
     delete $self->{'helo_name'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - SenderID Module
+Mail::Milter::Authentication::Handler::SenderID - Handler class for SenderID
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -148,17 +154,15 @@ Implements the SenderID standard checks.
             "hide_none" : 1                             | Hide auth line if the result is 'none'
         },
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
