@@ -1,12 +1,11 @@
 package Mail::Milter::Authentication::Handler::Size;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for message size metrics
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Data::Dumper;
-use English qw{ -no_match_vars };
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -22,6 +21,7 @@ sub grafana_rows {
 sub register_metrics {
     return {
         'size_total' => 'The number of emails processed for Size',
+        'size_header_bytes_added_total' => 'The header size added to emails processed for Size',
         'size_header_bytes_total' => 'The header size of emails processed for Size',
         'size_body_bytes_total' => 'The body size of emails processed for Size',
     };
@@ -31,46 +31,61 @@ sub envfrom_callback {
     my ( $self, $env_from )  = @_;
     $self->{'headersize'} = 0;
     $self->{'bodysize'} = 0;
-    return;
 }
 
 sub header_callback {
     my ( $self, $header, $value ) = @_;
     $self->{ 'headersize' } = $self->{ 'headersize' } + length( $header . ': ' . $value ) + 2;
-
-    return;
 }
 
 sub body_callback {
     my ( $self, $body_chunk ) = @_;
     $self->{ 'bodysize' } = $self->{ 'bodysize' } + length ( $body_chunk );
-    return;
 }
 
 sub eom_callback {
     my ($self) = @_;
-
     $self->metric_count( 'size_total', {}, 1 );
     $self->metric_count( 'size_header_bytes_total', {}, $self->{ 'headersize' } );
     $self->metric_count( 'size_body_bytes_total', {}, $self->{ 'bodysize' } );
-
-    return;
 }
 
 sub close_callback {
     my ( $self ) = @_;
+
+    my $top_handler = $self->get_top_handler();
+    if ( exists( $top_handler->{'pre_headers'} ) ) {
+        foreach my $header ( @{ $top_handler->{'pre_headers'} } ) {
+            my $size = length( $header->{'field'} ) + length ( $header->{'value'} ) + 3;
+            $self->metric_count( 'size_header_bytes_added_total', { where => 'pre', 'header' => $header->{'field'} }, $size );
+        }
+    }
+    if ( exists( $top_handler->{'add_headers'} ) ) {
+        foreach my $header ( @{ $top_handler->{'add_headers'} } ) {
+            my $size = length( $header->{'field'} ) + length ( $header->{'value'} ) + 3;
+            $self->metric_count( 'size_header_bytes_added_total', { where => 'add', 'header' => $header->{'field'} }, $size );
+        }
+    }
+
     delete $self->{'bodysize'};
     delete $self->{'headersize'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - Size Module
+Mail::Milter::Authentication::Handler::Size - Handler class for message size metrics
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -80,17 +95,15 @@ Module to provide metrics related to message size.
 
         "Size" : {}, | Config for the Size Module
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
