@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Handler::TLS;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for TLS
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {};
@@ -29,7 +30,12 @@ sub pre_loop_setup {
     if ( $protocol eq 'smtp' ) {
         warn 'When in smtp mode, the TLS handler requires the MTA to write TLS data into the first Received header.';
     }
-    return;
+}
+
+sub connect_callback {
+    my ( $self ) = @_;
+    # Reset state on a connection
+    delete $self->{'is_encrypted'};
 }
 
 sub envfrom_callback {
@@ -48,18 +54,19 @@ sub envfrom_callback {
 
     if ($version) {
         $self->dbgout( 'EncryptedAs', "$version, $cipher, $bits bits$trusted", LOG_INFO );
+        $self->{'is_encrypted'} = 1;
 
-        my $header = q{};
         my $metric_data = {};
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-tls' )->safe_set_value( 'pass' );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.version' )->safe_set_value( $version ) );
+        $metric_data->{ 'version' } = $version;
 
-        $header .= $self->format_header_entry( 'x-tls', 'pass' ) . ' ';
-        $header .= $self->format_header_entry( 'version', $version );
         if ( $cipher ) {
-            $header .= ' ' . $self->format_header_entry( 'cipher', $cipher );
+            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.cipher' )->safe_set_value( $cipher ) );
             $metric_data->{ 'cipher' } = $cipher;
         }
         if ( $bits ) {
-            $header .= ' ' . $self->format_header_entry( 'bits', $bits );
+            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.bits' )->safe_set_value( $bits ) );
             $metric_data->{ 'bits' } = $bits;
         }
         $metric_data->{ 'trusted' } = $trusted ? 1 : 0;
@@ -68,7 +75,9 @@ sub envfrom_callback {
 
         $self->add_auth_header( $header );
     }
-    return;
+    else {
+        $self->{'is_encrypted'} = 0;
+    }
 }
 
 sub header_callback {
@@ -98,18 +107,19 @@ sub header_callback {
 
     if ($version) {
         $self->dbgout( 'EncryptedAs', "$version, $cipher, $bits bits", LOG_INFO );
+        $self->{'is_encrypted'} = 1;
 
-        my $header = q{};
         my $metric_data = {};
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-tls' )->safe_set_value( 'pass' );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.version' )->safe_set_value( $version ) );
+        $metric_data->{ 'version' } = $version;
 
-        $header .= $self->format_header_entry( 'x-tls', 'pass' ) . ' ';
-        $header .= $self->format_header_entry( 'version', $version );
         if ( $cipher ) {
-            $header .= ' ' . $self->format_header_entry( 'cipher', $cipher );
+            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.cipher' )->safe_set_value( $cipher ) );
             $metric_data->{ 'cipher' } = $cipher;
         }
         if ( $bits ) {
-            $header .= ' ' . $self->format_header_entry( 'bits', $bits );
+            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'smtp.bits' )->safe_set_value( $bits ) );
             $metric_data->{ 'bits' } = $bits;
         }
 
@@ -117,23 +127,37 @@ sub header_callback {
 
         $self->add_auth_header( $header );
     }
+}
 
-    return;
+sub eoh_callback {
+    my ( $self ) = @_;
+    my $protocol = Mail::Milter::Authentication::Config::get_config()->{'protocol'};
+    return if $protocol ne 'smtp';
+    return if defined $self->{'is_encrypted'};
+    $self->{'is_encrypted'} = 0;
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'first_header_read'};
-    return;
+    delete $self->{'is_encrypted'};
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - TLS Module
+Mail::Milter::Authentication::Handler::TLS - Handler class for TLS
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -143,17 +167,15 @@ Identify TLS protected connections.
 
 No configuration options exist for this handler.
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
