@@ -1,11 +1,12 @@
 package Mail::Milter::Authentication::Handler::TrustedIP;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for Trusted IP addresses
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
 use Net::IP;
-use Sys::Syslog qw{:standard :macros};
 
 sub default_config {
     return {
@@ -52,26 +53,33 @@ sub connect_callback {
     $self->{'is_trusted_ip_address'} = 0;
     if ( $self->is_trusted_ip_address($ip) ) {
         $self->dbgout( 'TrustedIP', 'pass', LOG_DEBUG );
-        $self->add_c_auth_header('x-trusted-ip=pass');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-trusted-ip' )->safe_set_value( 'pass' );
+        $self->add_c_auth_header( $header );
         $self->{'is_trusted_ip_address'} = 1;
         $self->metric_count( 'trustedip_connect_total' );
     }
-    return;
 }
 
 sub close_callback {
     my ( $self ) = @_;
     delete $self->{'is_trusted_ip_address'};
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - TrustedIP Module
+Mail::Milter::Authentication::Handler::TrustedIP - Handler class for Trusted IP addresses
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -88,17 +96,15 @@ Detect a trusted IP address and act accordingly.
             ],
         },
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
