@@ -1,16 +1,14 @@
 package Mail::Milter::Authentication::Handler::XGoogleDKIM;
+use 5.20.0;
 use strict;
 use warnings;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Handler class for Google specific DKIM
+our $VERSION = '2.20200329.1'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
-use Data::Dumper;
-use English qw{ -no_match_vars };
-use Sys::Syslog qw{:standard :macros};
-
-use Mail::DKIM;
-use Mail::DKIM::Verifier;
 use Mail::DKIM::DNS;
+use Mail::DKIM::Verifier;
+use Mail::DKIM;
 
 sub default_config {
     return {
@@ -38,14 +36,13 @@ sub envfrom_callback {
     $self->{'has_dkim'}     = 0;
     $self->{'carry'}        = q{};
     $self->destroy_object('xgdkim');
-    return;
 }
 
 sub header_callback {
-    my ( $self, $header, $value ) = @_;
+    my ( $self, $header, $value, $original ) = @_;
     return if ( $self->{'failmode'} );
     my $EOL        = "\015\012";
-    my $dkim_chunk = $header . ': ' . $value . $EOL;
+    my $dkim_chunk = $original . $EOL;
     $dkim_chunk =~ s/\015?\012/$EOL/g;
 
     if ( lc($header) eq 'dkim-signature' ) {
@@ -63,9 +60,16 @@ sub header_callback {
         $x_dkim_chunk =~ s/\015?\012/$EOL/g;
         push @{$self->{'headers'}} , $x_dkim_chunk;
         $self->{'has_dkim'} = 1;
+        my ($domain) = $value =~ /d=([^;]*);/;
+        my ($selector) = $value =~ /s=([^;]*);/;
+        my $resolver = $self->get_object('resolver');
+        if ( $selector && $domain ) {
+            my $lookup = $selector.'._domainkey.'.$domain;
+            eval{ $resolver->bgsend( $lookup, 'TXT' ) };
+            $self->handle_exception( $@ );
+            $self->dbgout( 'DNSEarlyLookup', "$lookup TXT", LOG_DEBUG );
+        }
     }
-
-    return;
 }
 
 sub eoh_callback {
@@ -77,9 +81,9 @@ sub eoh_callback {
         $self->metric_count( 'xgoogledkim_total', { 'result' => 'none' } );
         $self->dbgout( 'XGoogleDKIMResult', 'No X-Google-DKIM headers', LOG_INFO );
         if ( !( $config->{'hide_none'} ) ) {
-            $self->add_auth_header(
-                $self->format_header_entry( 'x-google-dkim', 'none' )
-                . ' (no signatures found)' );
+            my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( 'none' );
+            $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'no signatures found' ) );
+            $self->add_auth_header( $header );
         }
         delete $self->{'headers'};
     }
@@ -96,10 +100,11 @@ sub eoh_callback {
             $self->set_object('xgdkim', $dkim, 1);
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'XGoogleDKIM Setup Error ' . $error );
+            $self->{'failmode'} = 1;
             $self->_check_error( $error );
             $self->metric_count( 'xgoogledkim_total', { 'result' => 'error' } );
-            $self->{'failmode'} = 1;
             delete $self->{'headers'};
             return;
         }
@@ -111,18 +116,17 @@ sub eoh_callback {
             );
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'XGoogleDKIM Headers Error ' . $error );
+            $self->{'failmode'} = 1;
             $self->_check_error( $error );
             $self->metric_count( 'xgoogledkim_total', { 'result' => 'error' } );
-            $self->{'failmode'} = 1;
         }
 
         delete $self->{'headers'};
     }
 
     $self->{'carry'} = q{};
-
-    return;
 }
 
 sub body_callback {
@@ -152,12 +156,12 @@ sub body_callback {
         $dkim->PRINT( $dkim_chunk );
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'XGoogleDKIM Body Error ' . $error );
+        $self->{'failmode'} = 1;
         $self->_check_error( $error );
         $self->metric_count( 'xgoogledkim_total', { 'result' => 'error' } );
-        $self->{'failmode'} = 1;
     }
-    return;
 }
 
 sub eom_callback {
@@ -176,16 +180,16 @@ sub eom_callback {
 
         my $dkim_result        = $dkim->result;
         my $dkim_result_detail = $dkim->result_detail;
-        
+
         $self->metric_count( 'xgoogledkim_total', { 'result' => $dkim_result } );
 
         $self->dbgout( 'XGoogleDKIMResult', $dkim_result_detail, LOG_INFO );
 
         if ( !$dkim->signatures() ) {
             if ( !( $config->{'hide_none'} && $dkim_result eq 'none' ) ) {
-                $self->add_auth_header(
-                    $self->format_header_entry( 'x-google-dkim', $dkim_result )
-                      . ' (no signatures found)' );
+                my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( $dkim_result );
+                $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'no signatures found' ) );
+                $self->add_auth_header( $header );
             }
         }
         foreach my $signature ( $dkim->signatures() ) {
@@ -214,7 +218,7 @@ sub eom_callback {
             if ( $signature_result ne 'pass' and $signature_result ne 'none' ) {
                 $signature_result_detail =~ /$signature_result \((.*)\)/;
                 if ( $1 ) {
-                    $result_comment = $1 . '; ';
+                    $result_comment = $1 . ', ';
                 }
             }
             if (
@@ -230,31 +234,23 @@ sub eom_callback {
                     $key_data = $key->size() . '-bit ' . $key->type() . ' key';
                 };
 
-                my $header = join(
-                    q{ },
-                    $self->format_header_entry( 'x-google-dkim', $signature_result ),
-                    '('
-                      . $self->format_header_comment(
-                        $result_comment
-                        . $key_data
-                      )
-                      . ')',
-                    $self->format_header_entry( 'header.d', $signature->domain() ),
-                    $self->format_header_entry( 'header.i', $signature->identity() ),
-                    $self->format_header_entry( 'header.b', substr( $signature->data(), 0, 8 ) ),
-                );
+                my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( $signature_result );
+                $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $result_comment . $key_data ) );
+                $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.d' )->safe_set_value( $signature->domain() ) );
+                $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.i' )->safe_set_value( $signature->identity() ) );
+                $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.b' )->safe_set_value( substr( $signature->data(), 0, 8 ) ) );
                 $self->add_auth_header($header);
             }
         }
 
     };
     if ( my $error = $@ ) {
-
+        $self->handle_exception( $error );
         # Also in DMARC module
         $self->log_error( 'XGoogleDKIM EOM Error ' . $error );
+        $self->{'failmode'} = 1;
         $self->_check_error( $error );
         $self->metric_count( 'xgoogledkim_total', { 'result' => 'error' } );
-        $self->{'failmode'} = 1;
         return;
     }
 }
@@ -267,7 +263,6 @@ sub close_callback {
     delete $self->{'carry'};
     delete $self->{'has_dkim'};
     $self->destroy_object('xgdkim');
-    return;
 }
 
 sub _check_error {
@@ -276,7 +271,9 @@ sub _check_error {
             or $error =~ /^DNS query timeout/
     ){
         $self->log_error( 'Temp XGoogleDKIM Error - ' . $error );
-        $self->add_auth_header('x-google-dkim=temperror (dns timeout)');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( 'temperror' );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'dns timeout' ) );
+        $self->add_auth_header( $header );
     }
     elsif ( $error =~ /^no domain to fetch policy for$/
             or $error =~ /^policy syntax error$/
@@ -284,26 +281,35 @@ sub _check_error {
             or $error =~ /^invalid name /
     ){
         $self->log_error( 'Perm XGoogleDKIM Error - ' . $error );
-        $self->add_auth_header('x-google-dkim=permerror (syntax or domain error)');
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( 'perlerror' );
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( 'syntax or domain error' ) );
+        $self->add_auth_header( $header );
     }
     else {
-        $self->log_error( 'Unexpected XGoogleDKIM Error - ' . $error );
-        $self->add_auth_header('x-google-dkim=temperror');
+        $self->exit_on_close( 'Unexpected XGoogleDKIM Error - ' . $error );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-google-dkim' )->safe_set_value( 'temperror' );
+        $self->add_auth_header( $header );
         # Fill these in as they occur, but for unknowns err on the side of caution
         # and tempfail/exit
-        $self->exit_on_close();
         $self->tempfail_on_error();
     }
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication-Milter - X Google DKIM Module
+Mail::Milter::Authentication::Handler::XGoogleDKIM - Handler class for Google specific DKIM
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -315,17 +321,15 @@ Module for validation of X-Google-DKIM signatures.
             "hide_none"         : 0,                    | Hide auth line if the result is 'none'
         },
 
-=head1 SYNOPSIS
+=head1 AUTHOR
 
-=head1 AUTHORS
+Marc Bradshaw <marc@marcbradshaw.net>
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+=head1 COPYRIGHT AND LICENSE
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
