@@ -1,9 +1,10 @@
 package Mail::Milter::Authentication::Metric::Grafana;
+use 5.20.0;
 use strict;
 use warnings;
-use version; our $VERSION = version->declare('v1.1.7');
-
-use JSON;
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Class for Grafana dashboards
+our $VERSION = '2.20200329.1'; # VERSION
 
 sub get_json {
     my ( $self, $file ) = @_;
@@ -62,9 +63,17 @@ sub get_dashboard {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Mail::Milter::Authentication::Metric::Grafana - Automatically generate Grafana dashboard
+Mail::Milter::Authentication::Metric::Grafana - Class for Grafana dashboards
+
+=head1 VERSION
+
+version 2.20200329.1
 
 =head1 DESCRIPTION
 
@@ -122,14 +131,15 @@ $server is the current handler object
 
 =back
 
-=head1 AUTHORS
+=head1 AUTHOR
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2017
+This software is copyright (c) 2020 by Marc Bradshaw.
 
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
