@@ -1,13 +1,12 @@
 package Mail::Milter::Authentication::Protocol::Milter;
+use 5.20.0;
 use strict;
 use warnings;
-use version; our $VERSION = version->declare('v1.1.7');
-
-use English qw{ -no_match_vars };
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Milter protocol handling
+our $VERSION = '2.20200329.1'; # VERSION
 use Net::IP;
 
-use Mail::Milter::Authentication::Constants qw{ :all };
-
 sub register_metrics {
     return {
         'mail_processed_total' => 'Number of emails processed',
@@ -41,8 +40,6 @@ sub protocol_process_request {
         $self->milter_process_command( $command, $data );
 
     }
-
-    return;
 }
 
 sub milter_process_command {
@@ -55,7 +52,8 @@ sub milter_process_command {
 
     if ( $command eq SMFIC_CONNECT ) {
         my ( $host, $ip ) = $self->milter_process_connect( $buffer );
-        $returncode = $handler->top_connect_callback( $host, $ip );
+        $handler->remap_connect_callback( $host, $ip );
+        $returncode = $handler->top_connect_callback( $host, $handler->{'ip_object'} );
     }
     elsif ( $command eq SMFIC_ABORT ) {
         $returncode = $handler->top_abort_callback();
@@ -82,11 +80,17 @@ sub milter_process_command {
     }
     elsif ( $command eq SMFIC_HELO ) {
         my $helo = $self->milter_split_buffer( $buffer );
-        $returncode = $handler->top_helo_callback( @$helo );
+        $handler->remap_helo_callback( @$helo );
+        $returncode = $handler->top_helo_callback( $handler->{'helo_name'} );
     }
     elsif ( $command eq SMFIC_HEADER ) {
         my $header = $self->milter_split_buffer( $buffer );
         if ( @$header == 1 ) { push @$header , q{}; };
+        my $original = join( ':', @$header );
+        push @$header, $original;
+        $header->[1] =~ s/^\s+//;
+        $header->[0] =~ s/^\s+//;
+        $header->[0] =~ s/\s+$//;
         $returncode = $handler->top_header_callback( @$header );
     }
     elsif ( $command eq SMFIC_MAIL ) {
@@ -121,14 +125,34 @@ sub milter_process_command {
         $self->fatal("Unknown milter command $command");
     }
 
+    my $config = $self->{'config'};
+
     my $reject_reason;
+    my $defer_reason;
+    my $quarantine_reason;
     if ( $reject_reason = $handler->get_reject_mail() ) {
         $handler->clear_reject_mail();
         $returncode = SMFIS_REJECT;
     }
+    elsif ( $defer_reason = $handler->get_defer_mail() ) {
+        $handler->clear_defer_mail();
+        $returncode = SMFIS_TEMPFAIL;
+    }
+    elsif ( $quarantine_reason = $handler->get_quarantine_mail() ) {
+        if ( $config->{'milter_quarantine'} ) {
+            $handler->clear_quarantine_mail();
+            $returncode = SMFIR_QUARANTINE;
+        }
+        else {
+            undef $quarantine_reason;
+        }
+    }
 
     if (defined $returncode) {
-        if ( $returncode == SMFIS_CONTINUE ) {
+        if ( $returncode eq SMFIR_QUARANTINE ) {
+            # NOP
+        }
+        elsif ( $returncode == SMFIS_CONTINUE ) {
             $returncode = SMFIR_CONTINUE;
         }
         elsif ( $returncode == SMFIS_TEMPFAIL ) {
@@ -145,7 +169,6 @@ sub milter_process_command {
             $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
         }
 
-        my $config = $self->{'config'};
         if ( $config->{'dryrun'} ) {
             if ( $returncode ne SMFIR_CONTINUE ) {
                 $self->loginfo ( "dryrun returncode changed from $returncode to continue" );
@@ -156,7 +179,7 @@ sub milter_process_command {
         if ( $command ne SMFIC_ABORT ) {
             if ( $reject_reason ) {
                 my ( $rcode, $xcode, $message ) = split( ' ', $reject_reason, 3 );
-                if ($rcode !~ /^[45]\d\d$/ || $xcode !~ /^[45]\.\d\.\d$/ || substr($rcode, 0, 1) ne substr($xcode, 0, 1)) {
+                if ($rcode !~ /^[5]\d\d$/ || $xcode !~ /^[5]\.\d\.\d$/ || substr($rcode, 0, 1) ne substr($xcode, 0, 1)) {
                     $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred_error' } );
                     $self->loginfo ( "Invalid reject message $reject_reason - setting to TempFail" );
                     $self->write_packet(SMFIR_TEMPFAIL );
@@ -170,13 +193,35 @@ sub milter_process_command {
                     );
                 }
             }
+            elsif ( $defer_reason ) {
+                my ( $rcode, $xcode, $message ) = split( ' ', $defer_reason, 3 );
+                if ($rcode !~ /^[4]\d\d$/ || $xcode !~ /^[4]\.\d\.\d$/ || substr($rcode, 0, 1) ne substr($xcode, 0, 1)) {
+                    $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred_error' } );
+                    $self->loginfo ( "Invalid defer message $defer_reason - setting to TempFail" );
+                    $self->write_packet(SMFIR_TEMPFAIL );
+                }
+                else {
+                    $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred' } );
+                    $self->loginfo ( "SMTPDefer: $reject_reason" );
+                    $self->write_packet( SMFIR_REPLYCODE,
+                        $defer_reason
+                        . "\0"
+                    );
+                }
+            }
+            elsif ( $quarantine_reason ) {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'quarantined' } );
+                $self->loginfo ( "SMTPQuarantine: $quarantine_reason" );
+                $self->write_packet( SMFIR_QUARANTINE,
+                    $quarantine_reason
+                    . "\0"
+                );
+            }
             else {
                 $self->write_packet($returncode);
             }
         }
     }
-
-    return;
 }
 
 sub milter_process_connect {
@@ -247,7 +292,6 @@ sub add_header {
         . $value
         . "\0"
     );
-    return;
 }
 
 sub change_header {
@@ -260,7 +304,6 @@ sub change_header {
         . $value
         . "\0"
     );
-    return;
 }
 
 sub insert_header {
@@ -272,7 +315,6 @@ sub insert_header {
         . $value
         . "\0"
     );
-    return;
 }
 
 sub write_packet {
@@ -284,20 +326,23 @@ sub write_packet {
     $socket->syswrite($len);
     $socket->syswrite($code);
     $socket->syswrite($data);
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Mail::Milter::Authentication::Protocol::Milter - Milter protocol specific methods
+Mail::Milter::Authentication::Protocol::Milter - Milter protocol handling
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-A Perl implementation of email authentication standards rolled up into a single easy to use milter.
+version 2.20200329.1
 
 =head1 SYNOPSIS
 
@@ -305,6 +350,10 @@ Subclass of Net::Server::PreFork for bringing up the main server process for aut
 
 Please see Net::Server docs for more detail of the server code.
 
+=head1 DESCRIPTION
+
+A Perl implementation of email authentication standards rolled up into a single easy to use milter.
+
 =head1 METHODS
 
 =over
@@ -356,19 +405,15 @@ Receive a new command from the protocol stream and process it.
 
 =back
 
-=head1 DEPENDENCIES
+=head1 AUTHOR
 
-  English
-  Net::IP
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 AUTHORS
+=head1 COPYRIGHT AND LICENSE
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+This software is copyright (c) 2020 by Marc Bradshaw.
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
