@@ -1,19 +1,17 @@
 package Mail::Milter::Authentication::Protocol::SMTP;
+use 5.20.0;
 use strict;
 use warnings;
-use version; our $VERSION = version->declare('v1.1.7');
-
-use English qw{ -no_match_vars };
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: SMTP protocol handling
+our $VERSION = '2.20200329.1'; # VERSION
+use Digest::MD5 qw{ md5_hex };
 use Email::Date::Format qw{ email_date };
-use IO::Socket;
+use File::Temp;
 use IO::Socket::INET;
 use IO::Socket::UNIX;
-use Digest::MD5 qw{ md5_hex };
+use IO::Socket;
 use Net::IP;
-use Sys::Syslog qw{:standard :macros};
-
-use Mail::Milter::Authentication::Constants qw{ :all };
-use Mail::Milter::Authentication::Config;
 
 sub register_metrics {
     return {
@@ -46,7 +44,6 @@ sub smtp_status {
     my ( $self, $status ) = @_;
     my $smtp = $self->{'smtp'};
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':' . $status . '(' . $self->{'count'} . '.' . $smtp->{'count'} . ')';
-    return;
 }
 
 sub smtp_init {
@@ -67,8 +64,6 @@ sub smtp_init {
     $handler->dbgout( 'SMTP Transaction count', $self->{'count'} . '.' . $smtp->{'count'} , LOG_INFO );
 
     $smtp->{'init_required'} = 0;
-
-    return;
 }
 
 sub protocol_process_request {
@@ -77,6 +72,10 @@ sub protocol_process_request {
     my $handler = $self->{'handler'}->{'_Handler'};
     $handler->top_setup_callback();
 
+    my $config = $self->{ 'config' };
+    my $seconds = $config->{'content_timeout'} // 300;
+    $handler->set_overall_timeout( $seconds * 1000000 );
+
     $self->{'smtp'} = {
         'fwd_helo_host'    => undef,
         'fwd_connect_ip'   => undef,
@@ -181,6 +180,7 @@ sub protocol_process_request {
             $self->smtp_command_rset( $command );
         }
         elsif ( $uccommand =~ /^DATA/ ) {
+            $handler->set_overall_timeout( $seconds * 1000000 );
             $self->smtp_command_data( $command );
         }
         elsif ( $uccommand =~ /^QUIT/ ){
@@ -195,13 +195,13 @@ sub protocol_process_request {
         }
 
     }
+    $handler->clear_overall_timeout();
 
     $self->smtp_status('smtp.close');
 
     $self->close_destination_socket();
 
     delete $self->{'smtp'};
-    return;
 }
 
 sub smtp_queue_id {
@@ -248,7 +248,6 @@ sub smtp_command_lhlo {
     print $socket "250-PIPELINING\r\n";
     print $socket "250-ENHANCEDSTATUSCODES\r\n";
     print $socket "250 8BITMIME\r\n";
-    return;
 }
 
 sub smtp_command_ehlo {
@@ -274,7 +273,6 @@ sub smtp_command_ehlo {
     print $socket "250-PIPELINING\r\n";
     print $socket "250-ENHANCEDSTATUSCODES\r\n";
     print $socket "250 8BITMIME\r\n";
-    return;
 }
 
 sub smtp_command_helo {
@@ -291,7 +289,6 @@ sub smtp_command_helo {
     }
     $smtp->{'helo_host'} = command_param( $command,5 );
     print $socket "250 " . $smtp->{'server_name'} . " Hi " . $smtp->{'helo_host'} . "\r\n";
-    return;
 }
 
 sub smtp_command_xforward {
@@ -337,7 +334,6 @@ sub smtp_command_xforward {
         }
     }
     print $socket "250 2.0.0 Ok\r\n";
-    return;
 }
 
 sub smtp_command_rset {
@@ -363,8 +359,15 @@ sub smtp_command_rset {
     $self->smtp_init();
 
     my $smtp_conf = $self->get_smtp_config();
+
     my $handler = $self->{'handler'}->{'_Handler'};
-        if ( $smtp_conf->{'pipeline_limit'} ) {
+    if ( $handler->{'exit_on_close'} ) {
+            $smtp->{'last_command'} = 1;
+            $handler->dbgout( 'EXIT ON CLOSE REQUESTED', 'closing on RSET', LOG_INFO );
+            print $socket "421 4.3.2 Pipeline limit reached\r\n";
+            return;
+    }
+    if ( $smtp_conf->{'pipeline_limit'} ) {
         my $count = $smtp->{'count'};
         my $limit = $smtp_conf->{'pipeline_limit'};
         if ( $count > $limit ) {
@@ -375,8 +378,6 @@ sub smtp_command_rset {
         }
     }
     print $socket "250 2.0.0 Ok\r\n";
-
-    return;
 }
 
 sub smtp_command_mailfrom {
@@ -407,10 +408,14 @@ sub smtp_command_mailfrom {
         $ip = substr( $ip, 5 );
     }
 
-    $self->logdebug( "Inbound IP Address $ip" );
-    $returncode = $handler->top_connect_callback( $host, Net::IP->new( $ip ) );
+    # Do connection remapping first
+    $handler->remap_connect_callback( $host, Net::IP->new( $ip ) );
+    $handler->remap_helo_callback( $helo );
+
+    $self->logdebug( "Inbound IP Address " . $handler->{'ip_object'}->ip() );
+    $returncode = $handler->top_connect_callback( $host, $handler->{'ip_object'} );
     if ( $returncode == SMFIS_CONTINUE ) {
-        $returncode = $handler->top_helo_callback( $helo );
+        $returncode = $handler->top_helo_callback( $handler->{'helo_name'} );
         if ( $returncode == SMFIS_CONTINUE ) {
             my $envfrom = command_param( $command,10 );
             $smtp->{'mail_from'} = $envfrom;
@@ -425,6 +430,11 @@ sub smtp_command_mailfrom {
                 $self->loginfo ( "SMTPReject: $reject_reason" );
                 print $socket $reject_reason . "\r\n";
             }
+            elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+                $handler->clear_defer_mail();
+                $self->loginfo ( "SMTPDefer: $defer_reason" );
+                print $socket $defer_reason . "\r\n";
+            }
             else {
                 print $socket "451 4.0.0 MAIL - That's not right\r\n";
             }
@@ -434,6 +444,11 @@ sub smtp_command_mailfrom {
             $self->loginfo ( "SMTPReject: $reject_reason" );
             print $socket $reject_reason . "\r\n";
         }
+        elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+            $handler->clear_defer_mail();
+            $self->loginfo ( "SMTPDefer: $defer_reason" );
+            print $socket $defer_reason . "\r\n";
+        }
         else {
             print $socket "451 4.0.1 HELO - That's not right\r\n";
         }
@@ -443,11 +458,14 @@ sub smtp_command_mailfrom {
         $self->loginfo ( "SMTPReject: $reject_reason" );
         print $socket $reject_reason . "\r\n";
     }
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->clear_defer_mail();
+        $self->loginfo ( "SMTPDefer: $defer_reason" );
+        print $socket $defer_reason . "\r\n";
+    }
     else {
         print $socket "451 4.0.2 Connection - That's not right\r\n";
     }
-
-    return;
 }
 
 sub smtp_command_rcptto {
@@ -474,11 +492,14 @@ sub smtp_command_rcptto {
         $self->loginfo ( "SMTPReject: $reject_reason" );
         print $socket $reject_reason . "\r\n";
     }
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->clear_defer_mail();
+        $self->loginfo ( "SMTPDefer: $defer_reason" );
+        print $socket $defer_reason . "\r\n";
+    }
     else {
         print $socket "451 4.0.3 That's not right\r\n";
     }
-
-    return;
 }
 
 sub smtp_command_data {
@@ -541,10 +562,14 @@ sub smtp_command_data {
             if ( $value ) {
                 push @{ $smtp->{'headers'} } , $value;
                 my ( $hkey, $hvalue ) = split ( ':', $value, 2 );
-                $hvalue =~ s/^ //;
-                my $returncode = $handler->top_header_callback( $hkey, $hvalue );
-                if ( $returncode != SMFIS_CONTINUE ) {
-                    $fail = 1;
+                $hvalue =~ s/^\s+//;
+                $hkey =~ s/^\s+//;
+                $hkey =~ s/\s+$//;
+                if ( ! $fail ) {
+                    my $returncode = $handler->top_header_callback( $hkey, $hvalue, $value );
+                    if ( $returncode != SMFIS_CONTINUE ) {
+                        $fail = 1;
+                    }
                 }
             }
             $value = $header_line;
@@ -553,18 +578,37 @@ sub smtp_command_data {
     if ( $value ) {
         push @{ $smtp->{'headers'} } , $value;
         my ( $hkey, $hvalue ) = split ( ':', $value, 2 );
-        $hvalue =~ s/^ //;
-        my $returncode = $handler->top_header_callback( $hkey, $hvalue );
+        $hvalue =~ s/^\s+//;
+        $hkey =~ s/^\s+//;
+        $hkey =~ s/\s+$//;
+        if ( ! $fail ) {
+            my $returncode = $handler->top_header_callback( $hkey, $hvalue, $value );
+            if ( $returncode != SMFIS_CONTINUE ) {
+                $fail = 1;
+            }
+        }
+    }
+    if ( ! $fail ) {
+        $returncode = $handler->top_eoh_callback();
         if ( $returncode != SMFIS_CONTINUE ) {
             $fail = 1;
         }
     }
-    $returncode = $handler->top_eoh_callback();
-    if ( $returncode != SMFIS_CONTINUE ) {
-        $fail = 1;
+
+    my $smtp_conf = $self->get_smtp_config();
+
+    my $chunk_limit = 1048576; # Process in chunks no larger than...
+    if ( exists ( $smtp_conf->{ 'chunk_limit' } ) ) {
+        $chunk_limit = $smtp_conf->{ 'chunk_limit' };
+    }
+
+    my $temp_file;
+    if ( exists ( $smtp_conf->{ 'temp_dir' } ) ) {
+        $temp_file = File::Temp->new( DIR => $smtp_conf->{ 'temp_dir' } );
     }
 
     $self->smtp_status('smtp.i.body');
+    my $body_chunk = q{};
     if ( ! $done ) {
         eval {
             alarm( $smtp->{'smtp_timeout_in'} );
@@ -577,12 +621,32 @@ sub smtp_command_data {
                 if ( $dataline =~ /^\./ ) {
                     $dataline = substr( $dataline, 1 );
                 }
-                $body .= $dataline;
+
+                if ( $temp_file ) {
+                    print $temp_file $dataline;
+                }
+                else {
+                   $body .= $dataline;
+                }
+
+                if ( length( $body_chunk ) + length( $dataline ) > $chunk_limit ) {
+                    $returncode = $handler->top_body_callback( $body_chunk );
+                    if ( $returncode != SMFIS_CONTINUE ) {
+                        $fail = 1;
+                    }
+                    $body_chunk = q{};
+                }
+
+                $body_chunk .= $dataline;
+
                 alarm( $smtp->{'smtp_timeout_in'} );
             }
-            $returncode = $handler->top_body_callback( $body );
-            if ( $returncode != SMFIS_CONTINUE ) {
-                $fail = 1;
+            if ( ! $fail ) {
+                $returncode = $handler->top_body_callback( $body_chunk );
+                if ( $returncode != SMFIS_CONTINUE ) {
+                    $fail = 1;
+                }
+                $body_chunk = q{};
             }
         };
         if ( my $error = $@ ) {
@@ -593,18 +657,33 @@ sub smtp_command_data {
         alarm( 0 );
     }
 
-    $returncode = $handler->top_eom_callback();
-    if ( $returncode != SMFIS_CONTINUE ) {
-        $fail = 1;
+    if ( ! $fail ) {
+        $returncode = $handler->top_eom_callback();
+        if ( $returncode != SMFIS_CONTINUE ) {
+            $fail = 1;
+        }
     }
+
     $self->smtp_status('smtp.i.data.received');
 
     if ( ! $fail ) {
-        $smtp->{'body'} = $body;
+
+        if ( $temp_file ) {
+            $smtp->{'spool'} = $temp_file;
+        }
+        else {
+            $smtp->{'body'} = $body;
+        }
 
         if ( $self->smtp_forward_to_destination() ) {
 
-            $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
+            if ( my $reject_reason = $handler->get_quarantine_mail() ) {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'quarantined' } );
+            }
+            else {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
+            }
+
             $handler->dbgout( 'Accept string', $smtp->{'string'}, LOG_INFO );
             $smtp->{'has_data'} = 1;
 
@@ -658,6 +737,20 @@ sub smtp_command_data {
             print $socket $reject_reason . "\r\n";
         }
     }
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->metric_count( 'mail_processed_total', { 'result' => 'defered' } );
+        $handler->clear_defer_mail();
+        if ( $smtp->{'using_lmtp'} ) {
+            foreach my $rcpt_to ( @{ $smtp->{'lmtp_rcpt'} } ) {
+                $self->loginfo ( "SMTPDefer: $defer_reason" );
+                print $socket $defer_reason . "\r\n";
+            }
+        }
+        else {
+            $self->loginfo ( "SMTPDefer: $defer_reason" );
+            print $socket $defer_reason . "\r\n";
+        }
+    }
     else {
         $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred_error' } );
         if ( $smtp->{'using_lmtp'} ) {
@@ -684,7 +777,11 @@ sub smtp_command_data {
     $smtp->{'string'}           = q{};
     $self->{'handler'}->{'_Handler'}->top_close_callback();
     $smtp->{'init_required'}    = 1;
-    return;
+
+    if ( $handler->{'exit_on_close'} ) {
+            $smtp->{'last_command'} = 1;
+            $handler->dbgout( 'EXIT ON CLOSE REQUESTED', 'NO MORE SMTP', LOG_INFO );
+    }
 }
 
 sub smtp_insert_received_header {
@@ -720,7 +817,6 @@ sub smtp_insert_received_header {
     );
 
     splice @{ $smtp->{'headers'} }, 0, 0, 'Received: '. $value;
-    return;
 }
 
 sub smtp_forward_to_destination {
@@ -825,14 +921,32 @@ sub smtp_forward_to_destination {
     }
     $email .= "\r\n";
 
-    my $body = $smtp->{'body'};
-    $body =~ s/\015?\012/\015\012/g;
-    $email .= $body;
+    my $spool = $smtp->{'spool'};
+    if ( $spool ) {
 
-    # Handle transparency
-    $email =~ s/\015\012\./\015\012\.\./g;
+        # Handle transparency - should not be any in headers, but for completeness
+        $email =~ s/\015\012\./\015\012\.\./g;
 
-    print $sock $email;
+        print $sock $email;
+
+        seek( $spool, 0, 0 );
+        while ( my $line = <$spool> ) {
+            $line =~ s/\015?\012/\015\012/g;
+            $line =~ s/^\./\.\./g;
+            print $sock $line;
+        }
+
+    }
+    else {
+        my $body = $smtp->{'body'};
+        $body =~ s/\015?\012/\015\012/g;
+        $email .= $body;
+
+        # Handle transparency
+        $email =~ s/\015\012\./\015\012\.\./g;
+
+        print $sock $email;
+    }
 
     $self->logdebug( 'Sending end to destination' );
     $self->send_smtp_packet( $sock, '.',    '250' ) || return;
@@ -850,7 +964,6 @@ sub close_destination_socket {
     $self->send_smtp_packet( $sock, 'QUIT', '221' ) || return;
     $sock->close();
     delete $smtp->{'destination_sock'};
-    return;
 }
 
 sub send_smtp_packet {
@@ -904,7 +1017,6 @@ sub add_header {
     my $smtp = $self->{'smtp'};
     $value =~ s/\015?\012/\015\012/g;
     push @{ $smtp->{'headers'} } , "$header: $value";
-    return;
 }
 
 sub change_header {
@@ -937,8 +1049,6 @@ sub change_header {
             #untested.
         }
     }
-
-    return;
 }
 
 sub insert_header {
@@ -946,20 +1056,23 @@ sub insert_header {
     my $smtp = $self->{'smtp'};
     $value =~ s/\015?\012/\015\012/g;
     splice @{ $smtp->{'headers'} }, $index - 1, 0, "$key: $value";
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Mail::Milter::Authentication::Protocol::SMTP - SMTP protocol specific methods
+Mail::Milter::Authentication::Protocol::SMTP - SMTP protocol handling
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-A Perl implenmetation of email authentication standards rolled up into a single easy to use milter.
+version 2.20200329.1
 
 =head1 SYNOPSIS
 
@@ -967,6 +1080,10 @@ Subclass of Net::Server::PreFork for bringing up the main server process for aut
 
 Please see Net::Server docs for more detail of the server code.
 
+=head1 DESCRIPTION
+
+A Perl implenmetation of email authentication standards rolled up into a single easy to use milter.
+
 =head1 FUNCTIONS
 
 =over
@@ -1078,21 +1195,15 @@ Update the process name status line
 
 =back
 
-=head1 DEPENDENCIES
+=head1 AUTHOR
 
-  English
-  Digest::MD5
-  Net::IP
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 AUTHORS
+=head1 COPYRIGHT AND LICENSE
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
