@@ -1,26 +1,23 @@
 package Mail::Milter::Authentication::Tester;
+use 5.20.0;
 use strict;
 use warnings;
-
-use Exporter;
-our @ISA = qw{ Exporter }; ## no critic
-our @EXPORT = qw{ start_milter stop_milter get_metrics test_metrics smtp_process smtp_process_multi milter_process smtpput send_smtp_packet smtpcat }; ## no critic
-
-use Net::DNS::Resolver::Mock;
-use Test::More;
-use Test::File::Contents;
-
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: Class used for testing
+our $VERSION = '2.20200329.1'; # VERSION
+use Mail::Milter::Authentication;
+use Mail::Milter::Authentication::Client;
+use Mail::Milter::Authentication::Protocol::Milter;
+use Mail::Milter::Authentication::Protocol::SMTP;
 use Cwd qw{ cwd };
 use IO::Socket::INET;
 use IO::Socket::UNIX;
-use JSON;
-use Module::Load;
+use Net::DNS::Resolver::Mock 1.20171219;
+use Test::File::Contents;
+use Test::More;
 
-use Mail::Milter::Authentication;
-use Mail::Milter::Authentication::Client;
-use Mail::Milter::Authentication::Config;
-use Mail::Milter::Authentication::Protocol::Milter;
-use Mail::Milter::Authentication::Protocol::SMTP;
+our @ISA = qw{ Exporter }; ## no critic
+our @EXPORT = qw{ start_milter stop_milter get_metrics test_metrics smtp_process smtp_process_multi milter_process smtpput send_smtp_packet smtpcat }; ## no critic
 
 my $base_dir = cwd();
 
@@ -131,13 +128,19 @@ sub test_metrics {
 
             foreach my $key ( sort keys %$data ) {
                 if ( $key =~ /seconds_total/ ) {
-                    is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $key" );
+                    is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $expected $key" );
                 }
                 elsif ( $key =~ /microseconds_sum/ ) {
-                    is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $key" );
+                    is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $expected $key" );
+                }
+                elsif ( $key =~ /authmilter_forked_children_total/ ) {
+                    is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $expected $key" );
+                }
+                elsif ( $key =~ /authmilter_processes_/) {
+                    is( $metrics->{ $key } > -1, $data->{ $key } > -1, "Metrics $expected $key" );
                 }
                 else {
-                    is( $metrics->{ $key }, $data->{ $key }, "Metrics $key" );
+                    is( $metrics->{ $key }, $data->{ $key }, "Metrics $expected $key" );
                 }
             }
 
@@ -154,6 +157,12 @@ sub test_metrics {
                 elsif ( $key =~ /microseconds_sum/ ) {
                     $metrics->{ $key } = 123456 if $metrics->{ $key } > 0;
                 }
+                elsif ( $key =~ /authmilter_forked_children_total/ ) {
+                    $metrics->{ $key } = 123456 if $metrics->{ $key } > 0;
+                }
+                elsif ( $key =~ /authmilter_processes_/) {
+                    $metrics->{ $key } = 123456 if $metrics->{ $key } > -1;
+                }
             }
             open my $OutF, '>', $expected;
             $j->pretty();
@@ -163,8 +172,6 @@ sub test_metrics {
         }
 
     };
-
-    return;
 }
 
 sub smtp_process {
@@ -205,13 +212,11 @@ sub smtp_process {
 
     if ( ! $args->{'no_cat'} ) {
         waitpid( $cat_pid,0 );
-        files_eq( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'smtp ' . $args->{'desc'} );
+        files_eq_or_diff( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'smtp ' . $args->{'desc'} );
     }
     else {
         is( $return, 1, 'SMTP Put Returned ok' );
     }
-
-    return;
 }
 
 sub smtp_process_multi {
@@ -272,9 +277,7 @@ sub smtp_process_multi {
 
     waitpid( $cat_pid,0 );
 
-    files_eq( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'smtp ' . $args->{'desc'} );
-
-    return;
+    files_eq_or_diff( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'smtp ' . $args->{'desc'} );
 }
 
 sub milter_process {
@@ -299,9 +302,7 @@ sub milter_process {
         'output'       => 'tmp/result/' . $args->{'dest'},
     });
 
-    files_eq( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'milter ' . $args->{'desc'} );
-
-    return;
+    files_eq_or_diff( 'data/example/' . $args->{'dest'}, 'tmp/result/' . $args->{'dest'}, 'milter ' . $args->{'desc'} );
 }
 
 sub smtpput {
@@ -556,7 +557,33 @@ sub client {
 
     }
     waitpid( $pid, 0 );
-    return;
 }
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Mail::Milter::Authentication::Tester - Class used for testing
+
+=head1 VERSION
+
+version 2.20200329.1
+
+=head1 AUTHOR
+
+Marc Bradshaw <marc@marcbradshaw.net>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2020 by Marc Bradshaw.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
